
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>

#define tmax T
#define nx N
#define ny N
double ex[nx][ny +1];
double ey[nx +1][ny];
double hz[nx][ny];

void init_arrays()
{
    int i, j;
    for (i=0; i<nx+1; i++)  {
        for (j=0; j<ny; j++)  {
            ey[i][j] = 0;
        }
    }
    for (i=0; i<nx; i++)  {
        for (j=0; j<ny+1; j++)  {
            ex[i][j] = 0;
        }
    }
    for (j=0; j<ny; j++)  {
        ey[0][j] = ((double)j)/ny;
    }
    for (i=0; i<nx; i++)    {
        for (j=0; j<ny; j++)  {
            hz[i][j] = 0;
        }
    }
}

double rtclock()
{
  struct timezone tzp;
  struct timeval tp;
  int stat;
  gettimeofday (&tp, &tzp);
  return (tp.tv_sec + tp.tv_usec*1.0e-6);
}

int main()
{
  init_arrays();

  double annot_t_start=0, annot_t_end=0, annot_t_total=0;
  int annot_i;

  for (annot_i=0; annot_i<REPS; annot_i++)
  {
    annot_t_start = rtclock();
    
/*@ begin PerfTuning (
  def build {
    arg build_command = 'icc -O3 -openmp -lm';
  }

  def performance_counter {
    arg repetitions = 1;
  }  
  
  def performance_params {  
#   [4,8,16,32,64,128];
#   [1,4,8,16];

    param T1_1[] = [64];
    param T1_2[] = [64];
    param T1_3[] = [64];
    param T2_1[] = [1];
    param T2_2[] = [1];
    param T2_3[] = [1];

#    constraint c1 = (T1_1*T2_1<=1024 and T1_2*T2_2<=1024 and T1_3*T2_3<=1024);
    constraint c2 = (T1_1==T1_3 and T2_1==T2_3);
    
    param U1[] = [7];
    param U2[] = [2];
    param U3[] = [3];

    constraint c3 = (U1*U2*U3<=256);

    param PERM[] = [
#      [0,1,2],
      [0,2,1],
#      [1,0,2],
#      [1,2,0],
#      [2,0,1],
#      [2,1,0],

    ];

    param PAR[] = [False];
    param SCREP[] = [False];
    param IVEC[] = [True];
  }
  
  def search
  {
   arg algorithm = 'Exhaustive';
#   arg algorithm = 'Simplex';
#   arg total_runs = 1;
#   arg algorithm = 'Random';
#   arg time_limit = 10;
  }

  def input_params
  {
  let N=2000;
  param tmax[] = [500];
  param nx[] = [N];
  param ny[] = [N]; 
  }

  def input_vars
  {
  decl static double ex[nx][ny+1] = random;
  decl static double ey[nx+1][ny] = random;
  decl static double hz[nx][ny] = random;
  }
) @*/

/**-- (Generated by Orio) 
Best performance cost: 
  7.484850 
Tuned for specific problem sizes: 
  nx = 2000 
  ny = 2000 
  tmax = 500 
Best performance parameters: 
  IVEC = True 
  PAR = False 
  PERM = [0, 2, 1] 
  SCREP = False 
  T1_1 = 64 
  T1_2 = 64 
  T1_3 = 64 
  T2_1 = 1 
  T2_2 = 1 
  T2_3 = 1 
  U1 = 7 
  U2 = 2 
  U3 = 3 
--**/

   

register int i,j,k,t;  
register int c1t, c2t, c3t, c4t, c5t, c6t, c7t, c8t, c9t, c10t, c11t, c12t;  
register int newlb_c1, newlb_c2, newlb_c3, newlb_c4, newlb_c5, newlb_c6,  
  newlb_c7, newlb_c8, newlb_c9, newlb_c10, newlb_c11, newlb_c12;  
register int newub_c1, newub_c2, newub_c3, newub_c4, newub_c5, newub_c6,  
  newub_c7, newub_c8, newub_c9, newub_c10, newub_c11, newub_c12;  

/*@ begin PolySyn(  
  parallel = PAR; 
  tiles = [T1_1,T1_2,T1_3,T2_1,T2_2,T2_3];
  permut = PERM;
  unroll_factors = [U1,U2,U3]; 
  scalar_replace = SCREP; 
  vectorize = IVEC; 
 
  profiling_code = 'fdtd-2d_profiling.c'; 
  compile_cmd = 'gcc'; 
  compile_opts = '-lm'; 
  ) @*/#include <math.h>
#include <assert.h>
#define ceild(n,d)  ceil(((double)(n))/((double)(d)))
#define floord(n,d) floor(((double)(n))/((double)(d)))
#define max(x,y)    ((x) > (y)? (x) : (y))
#define min(x,y)    ((x) < (y)? (x) : (y))
 








 int c1, c2, c3, c4, c5, c6, c7, c8, c9;
/* polysyn start */
for (c1=0;c1<=floord(tmax-1,64);c1++) {
  for (c2=max(0,ceild(64*c1-63,64));c2<=min(floord(tmax+ny-1,64),floord(64*c1+ny+63,64));c2++) {
    for (c3=max(max(max(max(max(max(max(max(max(max(ceild(64*c1-63*ny-3843,4032),0),ceild(64*c2-ny-62,64)),ceild(64*c1-63,64)),ceild(64*c1-4032*c2-7875,4032)),ceild(64*c1-4032*c2-62*nx-3907,64)),ceild(64*c1-ny-61,64)),ceild(128*c1-64*c2-125,64)),ceild(64*c1-3968*c2-62*nx-ny-3843,64)),ceild(4096*c1-4032*c2-62*nx-3907,64)),ceild(4032*c1-3968*c2-62*nx-ny-3843,64));c3<=min(min(min(floord(64*c2+nx+62,64),floord(tmax+nx-1,64)),floord(64*c1+nx+63,64)),c1+64*c2+nx+62);c3++) {
      if ((c1 <= floord(64*c3-nx,64)) && (c2 <= floord(64*c3-nx+ny,64)) && (c3 >= ceild(nx,64))) {
        for (c8=max(64*c3-nx+1,64*c2);c8<=min(64*c3-nx+ny,64*c2+63);c8++) {
          hz[nx-1][-64*c3+c8+nx-1]=hz[nx-1][-64*c3+c8+nx-1]-((double)(7))/10*(ey[1+nx-1][-64*c3+c8+nx-1]+ex[nx-1][1+-64*c3+c8+nx-1]-ex[nx-1][-64*c3+c8+nx-1]-ey[nx-1][-64*c3+c8+nx-1]);
        }
      }
      if ((c1 <= floord(64*c2-ny,64)) && (c2 >= max(ceild(ny,64),ceild(64*c3-nx+ny+1,64)))) {
        for (c9=max(64*c2-ny+1,64*c3);c9<=min(64*c2+nx-ny,64*c3+63);c9++) {
          hz[-64*c2+c9+ny-1][ny-1]=hz[-64*c2+c9+ny-1][ny-1]-((double)(7))/10*(ey[1+-64*c2+c9+ny-1][ny-1]+ex[-64*c2+c9+ny-1][1+ny-1]-ex[-64*c2+c9+ny-1][ny-1]-ey[-64*c2+c9+ny-1][ny-1]);
        }
      }
      if ((c1 == c3) && (nx >= 2)) {
        for (c7=max(max(0,64*c2-ny+1),64*c3);c7<=min(min(min(64*c2-1,64*c3-nx+63),tmax-1),64*c2-ny+63);c7++) {
          for (c8=64*c2;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=c7+1;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((c1 == c3) && (nx >= 2) && (ny >= 2)) {
        for (c7=max(max(64*c2,0),64*c3);c7<=min(min(64*c3-nx+63,tmax-1),64*c2-ny+63);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=c7+1;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((c1 == c3) && (nx >= 2)) {
        for (c7=max(max(0,64*c3),64*c2-ny+64);c7<=min(min(tmax-1,64*c2-1),64*c3-nx+63);c7++) {
          for (c8=64*c2;c8<=64*c2+63;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
        }
      }
      if (c1 == c3) {
        for (c7=max(max(max(64*c3-nx+64,0),64*c2-ny+1),64*c3);c7<=min(min(min(64*c2-1,tmax-1),64*c3+62),64*c2-ny+63);c7++) {
          for (c8=64*c2;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=64*c3+63;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=c7+1;c9<=64*c3+63;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((c1 == c3) && (nx >= 2)) {
        for (c7=max(max(max(64*c2,0),64*c3),64*c2-ny+64);c7<=min(min(tmax-1,64*c2+62),64*c3-nx+63);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=64*c2+63;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
        }
      }
      if ((c1 == c3) && (ny >= 2)) {
        for (c7=max(max(max(64*c2,64*c3-nx+64),0),64*c3);c7<=min(min(tmax-1,64*c3+62),64*c2-ny+63);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=64*c3+63;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=64*c3+63;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=c7+1;c9<=64*c3+63;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if (c1 == c3) {
        for (c7=max(max(max(0,64*c3),64*c3-nx+64),64*c2-ny+64);c7<=min(min(tmax-1,64*c3+62),64*c2-1);c7++) {
          for (c8=64*c2;c8<=64*c2+63;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=64*c3+63;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
        }
      }
      if (c1 == c3) {
        for (c7=max(max(max(max(64*c2,64*c3-nx+64),0),64*c3),64*c2-ny+64);c7<=min(min(tmax-1,64*c3+62),64*c2+62);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=64*c3+63;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=64*c2+63;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=64*c3+63;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
        }
      }
      if ((c1 == c3) && (nx >= 2) && (ny == 1)) {
        for (c7=max(max(0,64*c2),64*c3);c7<=min(min(tmax-1,64*c3+62),64*c2+62);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=min(c7+nx-1,64*c3+63);c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c9=c7+1;c9<=min(64*c3+63,c7+nx);c9++) {
            hz[-c7+c9-1][0]=hz[-c7+c9-1][0]-((double)(7))/10*(ey[1+-c7+c9-1][0]+ex[-c7+c9-1][1+0]-ex[-c7+c9-1][0]-ey[-c7+c9-1][0]);
          }
        }
      }
      if ((c1 == c3) && (nx == 1)) {
        for (c7=max(max(0,64*c2-ny+1),64*c3);c7<=min(min(min(64*c2-1,tmax-1),64*c3+62),64*c2-ny+63);c7++) {
          for (c8=64*c2;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
          hz[0][ny-1]=hz[0][ny-1]-((double)(7))/10*(ey[1+0][ny-1]+ex[0][1+ny-1]-ex[0][ny-1]-ey[0][ny-1]);
        }
      }
      if ((c1 == c3) && (nx == 1) && (ny >= 2)) {
        for (c7=max(max(64*c2,0),64*c3);c7<=min(min(tmax-1,64*c3+62),64*c2-ny+63);c7++) {
          ey[0][0]=c7;
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
          hz[0][ny-1]=hz[0][ny-1]-((double)(7))/10*(ey[1+0][ny-1]+ex[0][1+ny-1]-ex[0][ny-1]-ey[0][ny-1]);
        }
      }
      if ((c1 == c3) && (nx == 1)) {
        for (c7=max(max(0,64*c3),64*c2-ny+64);c7<=min(min(tmax-1,64*c3+62),64*c2-1);c7++) {
          for (c8=64*c2;c8<=64*c2+63;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
        }
      }
      if ((c1 == c3) && (nx == 1)) {
        for (c7=max(max(max(64*c2,0),64*c3),64*c2-ny+64);c7<=min(min(tmax-1,64*c3+62),64*c2+62);c7++) {
          ey[0][0]=c7;
          for (c8=c7+1;c8<=64*c2+63;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
        }
      }
      if ((c1 == c3) && (nx == 1) && (ny == 1)) {
        for (c7=max(max(0,64*c2),64*c3);c7<=min(min(tmax-1,64*c2+62),64*c3+62);c7++) {
          ey[0][0]=c7;
          hz[0][0]=hz[0][0]-((double)(7))/10*(ey[1+0][0]+ex[0][1+0]-ex[0][0]-ey[0][0]);
        }
      }
      for (c7=max(max(max(0,64*c3-nx+1),64*c2-ny+1),64*c1);c7<=min(min(min(min(min(64*c2-1,64*c3-nx+63),64*c3-1),tmax-1),64*c2-ny+63),64*c1+63);c7++) {
        for (c8=64*c2;c8<=c7+ny-1;c8++) {
          for (c9=64*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
          hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
        }
        for (c9=64*c3;c9<=c7+nx;c9++) {
          hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
        }
      }
      if (ny >= 2) {
        for (c7=max(max(max(64*c2,0),64*c3-nx+1),64*c1);c7<=min(min(min(min(64*c3-nx+63,64*c3-1),tmax-1),64*c2-ny+63),64*c1+63);c7++) {
          for (c9=64*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            for (c9=64*c3;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=64*c3;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      for (c7=max(max(max(0,64*c1),64*c3-nx+1),64*c2-ny+64);c7<=min(min(min(min(tmax-1,64*c3-1),64*c1+63),64*c2-1),64*c3-nx+63);c7++) {
        for (c8=64*c2;c8<=64*c2+63;c8++) {
          for (c9=64*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
          hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
        }
      }
      for (c7=max(max(max(64*c3-nx+64,0),64*c2-ny+1),64*c1);c7<=min(min(min(min(64*c2-1,64*c3-1),tmax-1),64*c2-ny+63),64*c1+63);c7++) {
        for (c8=64*c2;c8<=c7+ny-1;c8++) {
          for (c9=64*c3;c9<=64*c3+63;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
        for (c9=64*c3;c9<=64*c3+63;c9++) {
          hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
        }
      }
      for (c7=max(max(max(max(64*c2,0),64*c1),64*c3-nx+1),64*c2-ny+64);c7<=min(min(min(min(tmax-1,64*c3-1),64*c2+62),64*c1+63),64*c3-nx+63);c7++) {
        for (c9=64*c3;c9<=c7+nx-1;c9++) {
          ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
        }
        for (c8=c7+1;c8<=64*c2+63;c8++) {
          for (c9=64*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
          hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
        }
      }
      if (ny >= 2) {
        for (c7=max(max(max(64*c2,64*c3-nx+64),0),64*c1);c7<=min(min(min(64*c3-1,tmax-1),64*c2-ny+63),64*c1+63);c7++) {
          for (c9=64*c3;c9<=64*c3+63;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            for (c9=64*c3;c9<=64*c3+63;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=64*c3;c9<=64*c3+63;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
/*@ begin Loop(
transform Composite(
permut = [['c7', 'c9', 'c8']],
  regtile = (['c7', 'c8', 'c9'],[7, 2, 3]),
  scalarreplace = (False, 'double'),
  vector = (True, ['ivdep','vector always']))

      for (c7=max(max(max(0,64*c1),64*c3-nx+64),64*c2-ny+64);c7<=min(min(min(tmax-1,64*c3-1),64*c1+63),64*c2-1);c7++) {
        for (c8=64*c2;c8<=64*c2+63;c8++) {
          for (c9=64*c3;c9<=64*c3+63;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
      }

) @*/{
  for (c7t=max(max(max(0,64*c1),64*c3-nx+64),64*c2-ny+64); c7t<=min(min(min(tmax-1,64*c3-1),64*c1+63),64*c2-1)-6; c7t=c7t+7) {
    for (c9t=64*c3; c9t<=64*c3+61; c9t=c9t+3) {
      register int cbv_1, cbv_2;
      cbv_1=64*c2;
      cbv_2=64*c2+62;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_1; c8t<=cbv_2; c8t=c8t+2) {
        ey[-c7t+c9t][-c7t+c8t]=ey[-c7t+c9t][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8t]-hz[-c7t+c9t-1][-c7t+c8t]);
        ey[-c7t+c9t][-c7t+(c8t+1)]=ey[-c7t+c9t][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9t][-c7t+(c8t+1)]-hz[-c7t+c9t-1][-c7t+(c8t+1)]);
        ey[-c7t+(c9t+1)][-c7t+c8t]=ey[-c7t+(c9t+1)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8t]-hz[-c7t+(c9t+1)-1][-c7t+c8t]);
        ey[-c7t+(c9t+1)][-c7t+(c8t+1)]=ey[-c7t+(c9t+1)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+(c8t+1)]-hz[-c7t+(c9t+1)-1][-c7t+(c8t+1)]);
        ey[-c7t+(c9t+2)][-c7t+c8t]=ey[-c7t+(c9t+2)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8t]-hz[-c7t+(c9t+2)-1][-c7t+c8t]);
        ey[-c7t+(c9t+2)][-c7t+(c8t+1)]=ey[-c7t+(c9t+2)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+(c8t+1)]-hz[-c7t+(c9t+2)-1][-c7t+(c8t+1)]);
        ey[-(c7t+1)+c9t][-(c7t+1)+c8t]=ey[-(c7t+1)+c9t][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8t]-hz[-(c7t+1)+c9t-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]=ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]=ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+2)+c9t][-(c7t+2)+c8t]=ey[-(c7t+2)+c9t][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8t]-hz[-(c7t+2)+c9t-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]=ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]=ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+3)+c9t][-(c7t+3)+c8t]=ey[-(c7t+3)+c9t][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8t]-hz[-(c7t+3)+c9t-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]=ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]=ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+4)+c9t][-(c7t+4)+c8t]=ey[-(c7t+4)+c9t][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8t]-hz[-(c7t+4)+c9t-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]=ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]=ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+5)+c9t][-(c7t+5)+c8t]=ey[-(c7t+5)+c9t][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8t]-hz[-(c7t+5)+c9t-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]=ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]=ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+6)+c9t][-(c7t+6)+c8t]=ey[-(c7t+6)+c9t][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8t]-hz[-(c7t+6)+c9t-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]=ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]=ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)]);
        ex[-c7t+c9t][-c7t+c8t]=ex[-c7t+c9t][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8t]-hz[-c7t+c9t][-c7t+c8t-1]);
        ex[-c7t+c9t][-c7t+(c8t+1)]=ex[-c7t+c9t][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9t][-c7t+(c8t+1)]-hz[-c7t+c9t][-c7t+(c8t+1)-1]);
        ex[-c7t+(c9t+1)][-c7t+c8t]=ex[-c7t+(c9t+1)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8t]-hz[-c7t+(c9t+1)][-c7t+c8t-1]);
        ex[-c7t+(c9t+1)][-c7t+(c8t+1)]=ex[-c7t+(c9t+1)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+(c8t+1)]-hz[-c7t+(c9t+1)][-c7t+(c8t+1)-1]);
        ex[-c7t+(c9t+2)][-c7t+c8t]=ex[-c7t+(c9t+2)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8t]-hz[-c7t+(c9t+2)][-c7t+c8t-1]);
        ex[-c7t+(c9t+2)][-c7t+(c8t+1)]=ex[-c7t+(c9t+2)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+(c8t+1)]-hz[-c7t+(c9t+2)][-c7t+(c8t+1)-1]);
        ex[-(c7t+1)+c9t][-(c7t+1)+c8t]=ex[-(c7t+1)+c9t][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8t]-hz[-(c7t+1)+c9t][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]=ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]=ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+2)+c9t][-(c7t+2)+c8t]=ex[-(c7t+2)+c9t][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8t]-hz[-(c7t+2)+c9t][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]=ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]=ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+3)+c9t][-(c7t+3)+c8t]=ex[-(c7t+3)+c9t][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8t]-hz[-(c7t+3)+c9t][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]=ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]=ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+4)+c9t][-(c7t+4)+c8t]=ex[-(c7t+4)+c9t][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8t]-hz[-(c7t+4)+c9t][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]=ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]=ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+5)+c9t][-(c7t+5)+c8t]=ex[-(c7t+5)+c9t][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8t]-hz[-(c7t+5)+c9t][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]=ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]=ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+6)+c9t][-(c7t+6)+c8t]=ex[-(c7t+6)+c9t][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8t]-hz[-(c7t+6)+c9t][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]=ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]=ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)-1]);
        hz[-c7t+c9t-1][-c7t+c8t-1]=hz[-c7t+c9t-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+c9t-1][-c7t+c8t-1]+ex[-c7t+c9t-1][1+-c7t+c8t-1]-ex[-c7t+c9t-1][-c7t+c8t-1]-ey[-c7t+c9t-1][-c7t+c8t-1]);
        hz[-c7t+c9t-1][-c7t+(c8t+1)-1]=hz[-c7t+c9t-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+c9t-1][-c7t+(c8t+1)-1]+ex[-c7t+c9t-1][1+-c7t+(c8t+1)-1]-ex[-c7t+c9t-1][-c7t+(c8t+1)-1]-ey[-c7t+c9t-1][-c7t+(c8t+1)-1]);
        hz[-c7t+(c9t+1)-1][-c7t+c8t-1]=hz[-c7t+(c9t+1)-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+(c9t+1)-1][-c7t+c8t-1]+ex[-c7t+(c9t+1)-1][1+-c7t+c8t-1]-ex[-c7t+(c9t+1)-1][-c7t+c8t-1]-ey[-c7t+(c9t+1)-1][-c7t+c8t-1]);
        hz[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]=hz[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]+ex[-c7t+(c9t+1)-1][1+-c7t+(c8t+1)-1]-ex[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]-ey[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]);
        hz[-c7t+(c9t+2)-1][-c7t+c8t-1]=hz[-c7t+(c9t+2)-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+(c9t+2)-1][-c7t+c8t-1]+ex[-c7t+(c9t+2)-1][1+-c7t+c8t-1]-ex[-c7t+(c9t+2)-1][-c7t+c8t-1]-ey[-c7t+(c9t+2)-1][-c7t+c8t-1]);
        hz[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]=hz[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]+ex[-c7t+(c9t+2)-1][1+-c7t+(c8t+1)-1]-ex[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]-ey[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]);
        hz[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+c9t-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+c9t-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+(c9t+1)-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+(c9t+1)-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+(c9t+2)-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+(c9t+2)-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+c9t-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+c9t-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+(c9t+1)-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+(c9t+1)-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+(c9t+2)-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+(c9t+2)-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+c9t-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+c9t-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+(c9t+1)-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+(c9t+1)-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+(c9t+2)-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+(c9t+2)-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+c9t-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+c9t-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+(c9t+1)-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+(c9t+1)-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+(c9t+2)-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+(c9t+2)-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+c9t-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+c9t-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+(c9t+1)-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+(c9t+1)-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+(c9t+2)-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+(c9t+2)-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+c9t-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+c9t-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+(c9t+1)-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+(c9t+1)-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+(c9t+2)-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+(c9t+2)-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]);
      }
      register int cbv_3;
      cbv_3=64*c2+63;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_3; c8=c8+1) {
        ey[-c7t+c9t][-c7t+c8]=ey[-c7t+c9t][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8]-hz[-c7t+c9t-1][-c7t+c8]);
        ey[-c7t+(c9t+1)][-c7t+c8]=ey[-c7t+(c9t+1)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8]-hz[-c7t+(c9t+1)-1][-c7t+c8]);
        ey[-c7t+(c9t+2)][-c7t+c8]=ey[-c7t+(c9t+2)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8]-hz[-c7t+(c9t+2)-1][-c7t+c8]);
        ey[-(c7t+1)+c9t][-(c7t+1)+c8]=ey[-(c7t+1)+c9t][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8]-hz[-(c7t+1)+c9t-1][-(c7t+1)+c8]);
        ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]=ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8]);
        ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]=ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8]);
        ey[-(c7t+2)+c9t][-(c7t+2)+c8]=ey[-(c7t+2)+c9t][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8]-hz[-(c7t+2)+c9t-1][-(c7t+2)+c8]);
        ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]=ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8]);
        ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]=ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8]);
        ey[-(c7t+3)+c9t][-(c7t+3)+c8]=ey[-(c7t+3)+c9t][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8]-hz[-(c7t+3)+c9t-1][-(c7t+3)+c8]);
        ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]=ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8]);
        ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]=ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8]);
        ey[-(c7t+4)+c9t][-(c7t+4)+c8]=ey[-(c7t+4)+c9t][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8]-hz[-(c7t+4)+c9t-1][-(c7t+4)+c8]);
        ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]=ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8]);
        ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]=ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8]);
        ey[-(c7t+5)+c9t][-(c7t+5)+c8]=ey[-(c7t+5)+c9t][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8]-hz[-(c7t+5)+c9t-1][-(c7t+5)+c8]);
        ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]=ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8]);
        ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]=ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8]);
        ey[-(c7t+6)+c9t][-(c7t+6)+c8]=ey[-(c7t+6)+c9t][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8]-hz[-(c7t+6)+c9t-1][-(c7t+6)+c8]);
        ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]=ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8]);
        ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]=ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8]);
        ex[-c7t+c9t][-c7t+c8]=ex[-c7t+c9t][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8]-hz[-c7t+c9t][-c7t+c8-1]);
        ex[-c7t+(c9t+1)][-c7t+c8]=ex[-c7t+(c9t+1)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8]-hz[-c7t+(c9t+1)][-c7t+c8-1]);
        ex[-c7t+(c9t+2)][-c7t+c8]=ex[-c7t+(c9t+2)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8]-hz[-c7t+(c9t+2)][-c7t+c8-1]);
        ex[-(c7t+1)+c9t][-(c7t+1)+c8]=ex[-(c7t+1)+c9t][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8]-hz[-(c7t+1)+c9t][-(c7t+1)+c8-1]);
        ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]=ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8-1]);
        ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]=ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8-1]);
        ex[-(c7t+2)+c9t][-(c7t+2)+c8]=ex[-(c7t+2)+c9t][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8]-hz[-(c7t+2)+c9t][-(c7t+2)+c8-1]);
        ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]=ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8-1]);
        ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]=ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8-1]);
        ex[-(c7t+3)+c9t][-(c7t+3)+c8]=ex[-(c7t+3)+c9t][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8]-hz[-(c7t+3)+c9t][-(c7t+3)+c8-1]);
        ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]=ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8-1]);
        ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]=ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8-1]);
        ex[-(c7t+4)+c9t][-(c7t+4)+c8]=ex[-(c7t+4)+c9t][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8]-hz[-(c7t+4)+c9t][-(c7t+4)+c8-1]);
        ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]=ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8-1]);
        ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]=ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8-1]);
        ex[-(c7t+5)+c9t][-(c7t+5)+c8]=ex[-(c7t+5)+c9t][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8]-hz[-(c7t+5)+c9t][-(c7t+5)+c8-1]);
        ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]=ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8-1]);
        ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]=ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8-1]);
        ex[-(c7t+6)+c9t][-(c7t+6)+c8]=ex[-(c7t+6)+c9t][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8]-hz[-(c7t+6)+c9t][-(c7t+6)+c8-1]);
        ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]=ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8-1]);
        ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]=ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8-1]);
        hz[-c7t+c9t-1][-c7t+c8-1]=hz[-c7t+c9t-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+c9t-1][-c7t+c8-1]+ex[-c7t+c9t-1][1+-c7t+c8-1]-ex[-c7t+c9t-1][-c7t+c8-1]-ey[-c7t+c9t-1][-c7t+c8-1]);
        hz[-c7t+(c9t+1)-1][-c7t+c8-1]=hz[-c7t+(c9t+1)-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+(c9t+1)-1][-c7t+c8-1]+ex[-c7t+(c9t+1)-1][1+-c7t+c8-1]-ex[-c7t+(c9t+1)-1][-c7t+c8-1]-ey[-c7t+(c9t+1)-1][-c7t+c8-1]);
        hz[-c7t+(c9t+2)-1][-c7t+c8-1]=hz[-c7t+(c9t+2)-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+(c9t+2)-1][-c7t+c8-1]+ex[-c7t+(c9t+2)-1][1+-c7t+c8-1]-ex[-c7t+(c9t+2)-1][-c7t+c8-1]-ey[-c7t+(c9t+2)-1][-c7t+c8-1]);
        hz[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9t-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+c9t-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]);
        hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+(c9t+1)-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]);
        hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+(c9t+2)-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]);
        hz[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9t-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+c9t-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]);
        hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+(c9t+1)-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]);
        hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+(c9t+2)-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]);
        hz[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9t-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+c9t-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]);
        hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+(c9t+1)-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]);
        hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+(c9t+2)-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]);
        hz[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9t-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+c9t-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]);
        hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+(c9t+1)-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]);
        hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+(c9t+2)-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]);
        hz[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9t-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+c9t-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]);
        hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+(c9t+1)-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]);
        hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+(c9t+2)-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]);
        hz[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9t-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+c9t-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]);
        hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+(c9t+1)-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]);
        hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+(c9t+2)-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]);
      }
    }
    for (c9=c9t; c9<=64*c3+63; c9=c9+1) {
      register int cbv_4, cbv_5;
      cbv_4=64*c2;
      cbv_5=64*c2+62;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_4; c8t<=cbv_5; c8t=c8t+2) {
        ey[-c7t+c9][-c7t+c8t]=ey[-c7t+c9][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8t]-hz[-c7t+c9-1][-c7t+c8t]);
        ey[-c7t+c9][-c7t+(c8t+1)]=ey[-c7t+c9][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9][-c7t+(c8t+1)]-hz[-c7t+c9-1][-c7t+(c8t+1)]);
        ey[-(c7t+1)+c9][-(c7t+1)+c8t]=ey[-(c7t+1)+c9][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8t]-hz[-(c7t+1)+c9-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+2)+c9][-(c7t+2)+c8t]=ey[-(c7t+2)+c9][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8t]-hz[-(c7t+2)+c9-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+3)+c9][-(c7t+3)+c8t]=ey[-(c7t+3)+c9][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8t]-hz[-(c7t+3)+c9-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+4)+c9][-(c7t+4)+c8t]=ey[-(c7t+4)+c9][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8t]-hz[-(c7t+4)+c9-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+5)+c9][-(c7t+5)+c8t]=ey[-(c7t+5)+c9][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8t]-hz[-(c7t+5)+c9-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+6)+c9][-(c7t+6)+c8t]=ey[-(c7t+6)+c9][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8t]-hz[-(c7t+6)+c9-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)]);
        ex[-c7t+c9][-c7t+c8t]=ex[-c7t+c9][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8t]-hz[-c7t+c9][-c7t+c8t-1]);
        ex[-c7t+c9][-c7t+(c8t+1)]=ex[-c7t+c9][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9][-c7t+(c8t+1)]-hz[-c7t+c9][-c7t+(c8t+1)-1]);
        ex[-(c7t+1)+c9][-(c7t+1)+c8t]=ex[-(c7t+1)+c9][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8t]-hz[-(c7t+1)+c9][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+2)+c9][-(c7t+2)+c8t]=ex[-(c7t+2)+c9][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8t]-hz[-(c7t+2)+c9][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+3)+c9][-(c7t+3)+c8t]=ex[-(c7t+3)+c9][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8t]-hz[-(c7t+3)+c9][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+4)+c9][-(c7t+4)+c8t]=ex[-(c7t+4)+c9][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8t]-hz[-(c7t+4)+c9][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+5)+c9][-(c7t+5)+c8t]=ex[-(c7t+5)+c9][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8t]-hz[-(c7t+5)+c9][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+6)+c9][-(c7t+6)+c8t]=ex[-(c7t+6)+c9][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8t]-hz[-(c7t+6)+c9][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9][-(c7t+6)+(c8t+1)-1]);
        hz[-c7t+c9-1][-c7t+c8t-1]=hz[-c7t+c9-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+c9-1][-c7t+c8t-1]+ex[-c7t+c9-1][1+-c7t+c8t-1]-ex[-c7t+c9-1][-c7t+c8t-1]-ey[-c7t+c9-1][-c7t+c8t-1]);
        hz[-c7t+c9-1][-c7t+(c8t+1)-1]=hz[-c7t+c9-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+c9-1][-c7t+(c8t+1)-1]+ex[-c7t+c9-1][1+-c7t+(c8t+1)-1]-ex[-c7t+c9-1][-c7t+(c8t+1)-1]-ey[-c7t+c9-1][-c7t+(c8t+1)-1]);
        hz[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+c9-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+c9-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+c9-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+c9-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+c9-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+c9-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+c9-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+c9-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+c9-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+c9-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+c9-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+c9-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]);
      }
      register int cbv_6;
      cbv_6=64*c2+63;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_6; c8=c8+1) {
        ey[-c7t+c9][-c7t+c8]=ey[-c7t+c9][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8]-hz[-c7t+c9-1][-c7t+c8]);
        ey[-(c7t+1)+c9][-(c7t+1)+c8]=ey[-(c7t+1)+c9][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8]-hz[-(c7t+1)+c9-1][-(c7t+1)+c8]);
        ey[-(c7t+2)+c9][-(c7t+2)+c8]=ey[-(c7t+2)+c9][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8]-hz[-(c7t+2)+c9-1][-(c7t+2)+c8]);
        ey[-(c7t+3)+c9][-(c7t+3)+c8]=ey[-(c7t+3)+c9][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8]-hz[-(c7t+3)+c9-1][-(c7t+3)+c8]);
        ey[-(c7t+4)+c9][-(c7t+4)+c8]=ey[-(c7t+4)+c9][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8]-hz[-(c7t+4)+c9-1][-(c7t+4)+c8]);
        ey[-(c7t+5)+c9][-(c7t+5)+c8]=ey[-(c7t+5)+c9][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8]-hz[-(c7t+5)+c9-1][-(c7t+5)+c8]);
        ey[-(c7t+6)+c9][-(c7t+6)+c8]=ey[-(c7t+6)+c9][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8]-hz[-(c7t+6)+c9-1][-(c7t+6)+c8]);
        ex[-c7t+c9][-c7t+c8]=ex[-c7t+c9][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8]-hz[-c7t+c9][-c7t+c8-1]);
        ex[-(c7t+1)+c9][-(c7t+1)+c8]=ex[-(c7t+1)+c9][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8]-hz[-(c7t+1)+c9][-(c7t+1)+c8-1]);
        ex[-(c7t+2)+c9][-(c7t+2)+c8]=ex[-(c7t+2)+c9][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8]-hz[-(c7t+2)+c9][-(c7t+2)+c8-1]);
        ex[-(c7t+3)+c9][-(c7t+3)+c8]=ex[-(c7t+3)+c9][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8]-hz[-(c7t+3)+c9][-(c7t+3)+c8-1]);
        ex[-(c7t+4)+c9][-(c7t+4)+c8]=ex[-(c7t+4)+c9][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8]-hz[-(c7t+4)+c9][-(c7t+4)+c8-1]);
        ex[-(c7t+5)+c9][-(c7t+5)+c8]=ex[-(c7t+5)+c9][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8]-hz[-(c7t+5)+c9][-(c7t+5)+c8-1]);
        ex[-(c7t+6)+c9][-(c7t+6)+c8]=ex[-(c7t+6)+c9][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8]-hz[-(c7t+6)+c9][-(c7t+6)+c8-1]);
        hz[-c7t+c9-1][-c7t+c8-1]=hz[-c7t+c9-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+c9-1][-c7t+c8-1]+ex[-c7t+c9-1][1+-c7t+c8-1]-ex[-c7t+c9-1][-c7t+c8-1]-ey[-c7t+c9-1][-c7t+c8-1]);
        hz[-(c7t+1)+c9-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+c9-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+c9-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+c9-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+c9-1][-(c7t+1)+c8-1]);
        hz[-(c7t+2)+c9-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+c9-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+c9-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+c9-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+c9-1][-(c7t+2)+c8-1]);
        hz[-(c7t+3)+c9-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+c9-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+c9-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+c9-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+c9-1][-(c7t+3)+c8-1]);
        hz[-(c7t+4)+c9-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+c9-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+c9-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+c9-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+c9-1][-(c7t+4)+c8-1]);
        hz[-(c7t+5)+c9-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+c9-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+c9-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+c9-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+c9-1][-(c7t+5)+c8-1]);
        hz[-(c7t+6)+c9-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+c9-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+c9-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+c9-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+c9-1][-(c7t+6)+c8-1]);
      }
    }
  }
  for (c7=c7t; c7<=min(min(min(tmax-1,64*c3-1),64*c1+63),64*c2-1); c7=c7+1) {
    for (c9t=64*c3; c9t<=64*c3+61; c9t=c9t+3) {
      register int cbv_7, cbv_8;
      cbv_7=64*c2;
      cbv_8=64*c2+62;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_7; c8t<=cbv_8; c8t=c8t+2) {
        ey[-c7+c9t][-c7+c8t]=ey[-c7+c9t][-c7+c8t]-((double)(1))/2*(hz[-c7+c9t][-c7+c8t]-hz[-c7+c9t-1][-c7+c8t]);
        ey[-c7+c9t][-c7+(c8t+1)]=ey[-c7+c9t][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9t][-c7+(c8t+1)]-hz[-c7+c9t-1][-c7+(c8t+1)]);
        ey[-c7+(c9t+1)][-c7+c8t]=ey[-c7+(c9t+1)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8t]-hz[-c7+(c9t+1)-1][-c7+c8t]);
        ey[-c7+(c9t+1)][-c7+(c8t+1)]=ey[-c7+(c9t+1)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+(c8t+1)]-hz[-c7+(c9t+1)-1][-c7+(c8t+1)]);
        ey[-c7+(c9t+2)][-c7+c8t]=ey[-c7+(c9t+2)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8t]-hz[-c7+(c9t+2)-1][-c7+c8t]);
        ey[-c7+(c9t+2)][-c7+(c8t+1)]=ey[-c7+(c9t+2)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+(c8t+1)]-hz[-c7+(c9t+2)-1][-c7+(c8t+1)]);
        ex[-c7+c9t][-c7+c8t]=ex[-c7+c9t][-c7+c8t]-((double)(1))/2*(hz[-c7+c9t][-c7+c8t]-hz[-c7+c9t][-c7+c8t-1]);
        ex[-c7+c9t][-c7+(c8t+1)]=ex[-c7+c9t][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9t][-c7+(c8t+1)]-hz[-c7+c9t][-c7+(c8t+1)-1]);
        ex[-c7+(c9t+1)][-c7+c8t]=ex[-c7+(c9t+1)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8t]-hz[-c7+(c9t+1)][-c7+c8t-1]);
        ex[-c7+(c9t+1)][-c7+(c8t+1)]=ex[-c7+(c9t+1)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+(c8t+1)]-hz[-c7+(c9t+1)][-c7+(c8t+1)-1]);
        ex[-c7+(c9t+2)][-c7+c8t]=ex[-c7+(c9t+2)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8t]-hz[-c7+(c9t+2)][-c7+c8t-1]);
        ex[-c7+(c9t+2)][-c7+(c8t+1)]=ex[-c7+(c9t+2)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+(c8t+1)]-hz[-c7+(c9t+2)][-c7+(c8t+1)-1]);
        hz[-c7+c9t-1][-c7+c8t-1]=hz[-c7+c9t-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+c9t-1][-c7+c8t-1]+ex[-c7+c9t-1][1+-c7+c8t-1]-ex[-c7+c9t-1][-c7+c8t-1]-ey[-c7+c9t-1][-c7+c8t-1]);
        hz[-c7+c9t-1][-c7+(c8t+1)-1]=hz[-c7+c9t-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+c9t-1][-c7+(c8t+1)-1]+ex[-c7+c9t-1][1+-c7+(c8t+1)-1]-ex[-c7+c9t-1][-c7+(c8t+1)-1]-ey[-c7+c9t-1][-c7+(c8t+1)-1]);
        hz[-c7+(c9t+1)-1][-c7+c8t-1]=hz[-c7+(c9t+1)-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+(c9t+1)-1][-c7+c8t-1]+ex[-c7+(c9t+1)-1][1+-c7+c8t-1]-ex[-c7+(c9t+1)-1][-c7+c8t-1]-ey[-c7+(c9t+1)-1][-c7+c8t-1]);
        hz[-c7+(c9t+1)-1][-c7+(c8t+1)-1]=hz[-c7+(c9t+1)-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+(c9t+1)-1][-c7+(c8t+1)-1]+ex[-c7+(c9t+1)-1][1+-c7+(c8t+1)-1]-ex[-c7+(c9t+1)-1][-c7+(c8t+1)-1]-ey[-c7+(c9t+1)-1][-c7+(c8t+1)-1]);
        hz[-c7+(c9t+2)-1][-c7+c8t-1]=hz[-c7+(c9t+2)-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+(c9t+2)-1][-c7+c8t-1]+ex[-c7+(c9t+2)-1][1+-c7+c8t-1]-ex[-c7+(c9t+2)-1][-c7+c8t-1]-ey[-c7+(c9t+2)-1][-c7+c8t-1]);
        hz[-c7+(c9t+2)-1][-c7+(c8t+1)-1]=hz[-c7+(c9t+2)-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+(c9t+2)-1][-c7+(c8t+1)-1]+ex[-c7+(c9t+2)-1][1+-c7+(c8t+1)-1]-ex[-c7+(c9t+2)-1][-c7+(c8t+1)-1]-ey[-c7+(c9t+2)-1][-c7+(c8t+1)-1]);
      }
      register int cbv_9;
      cbv_9=64*c2+63;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_9; c8=c8+1) {
        ey[-c7+c9t][-c7+c8]=ey[-c7+c9t][-c7+c8]-((double)(1))/2*(hz[-c7+c9t][-c7+c8]-hz[-c7+c9t-1][-c7+c8]);
        ey[-c7+(c9t+1)][-c7+c8]=ey[-c7+(c9t+1)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8]-hz[-c7+(c9t+1)-1][-c7+c8]);
        ey[-c7+(c9t+2)][-c7+c8]=ey[-c7+(c9t+2)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8]-hz[-c7+(c9t+2)-1][-c7+c8]);
        ex[-c7+c9t][-c7+c8]=ex[-c7+c9t][-c7+c8]-((double)(1))/2*(hz[-c7+c9t][-c7+c8]-hz[-c7+c9t][-c7+c8-1]);
        ex[-c7+(c9t+1)][-c7+c8]=ex[-c7+(c9t+1)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8]-hz[-c7+(c9t+1)][-c7+c8-1]);
        ex[-c7+(c9t+2)][-c7+c8]=ex[-c7+(c9t+2)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8]-hz[-c7+(c9t+2)][-c7+c8-1]);
        hz[-c7+c9t-1][-c7+c8-1]=hz[-c7+c9t-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9t-1][-c7+c8-1]+ex[-c7+c9t-1][1+-c7+c8-1]-ex[-c7+c9t-1][-c7+c8-1]-ey[-c7+c9t-1][-c7+c8-1]);
        hz[-c7+(c9t+1)-1][-c7+c8-1]=hz[-c7+(c9t+1)-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+(c9t+1)-1][-c7+c8-1]+ex[-c7+(c9t+1)-1][1+-c7+c8-1]-ex[-c7+(c9t+1)-1][-c7+c8-1]-ey[-c7+(c9t+1)-1][-c7+c8-1]);
        hz[-c7+(c9t+2)-1][-c7+c8-1]=hz[-c7+(c9t+2)-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+(c9t+2)-1][-c7+c8-1]+ex[-c7+(c9t+2)-1][1+-c7+c8-1]-ex[-c7+(c9t+2)-1][-c7+c8-1]-ey[-c7+(c9t+2)-1][-c7+c8-1]);
      }
    }
    for (c9=c9t; c9<=64*c3+63; c9=c9+1) {
      register int cbv_10, cbv_11;
      cbv_10=64*c2;
      cbv_11=64*c2+62;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_10; c8t<=cbv_11; c8t=c8t+2) {
        ey[-c7+c9][-c7+c8t]=ey[-c7+c9][-c7+c8t]-((double)(1))/2*(hz[-c7+c9][-c7+c8t]-hz[-c7+c9-1][-c7+c8t]);
        ey[-c7+c9][-c7+(c8t+1)]=ey[-c7+c9][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9][-c7+(c8t+1)]-hz[-c7+c9-1][-c7+(c8t+1)]);
        ex[-c7+c9][-c7+c8t]=ex[-c7+c9][-c7+c8t]-((double)(1))/2*(hz[-c7+c9][-c7+c8t]-hz[-c7+c9][-c7+c8t-1]);
        ex[-c7+c9][-c7+(c8t+1)]=ex[-c7+c9][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9][-c7+(c8t+1)]-hz[-c7+c9][-c7+(c8t+1)-1]);
        hz[-c7+c9-1][-c7+c8t-1]=hz[-c7+c9-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8t-1]+ex[-c7+c9-1][1+-c7+c8t-1]-ex[-c7+c9-1][-c7+c8t-1]-ey[-c7+c9-1][-c7+c8t-1]);
        hz[-c7+c9-1][-c7+(c8t+1)-1]=hz[-c7+c9-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+(c8t+1)-1]+ex[-c7+c9-1][1+-c7+(c8t+1)-1]-ex[-c7+c9-1][-c7+(c8t+1)-1]-ey[-c7+c9-1][-c7+(c8t+1)-1]);
      }
      register int cbv_12;
      cbv_12=64*c2+63;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_12; c8=c8+1) {
        ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
        ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
        hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
      }
    }
  }
}
/*@ end @*/

      for (c7=max(max(max(max(64*c2,64*c3-nx+64),0),64*c1),64*c2-ny+64);c7<=min(min(min(tmax-1,64*c3-1),64*c1+63),64*c2+62);c7++) {
        for (c9=64*c3;c9<=64*c3+63;c9++) {
          ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
        }
        for (c8=c7+1;c8<=64*c2+63;c8++) {
          for (c9=64*c3;c9<=64*c3+63;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
      }
      if (ny == 1) {
        for (c7=max(max(max(0,64*c3-nx+1),64*c1),64*c2);c7<=min(min(min(64*c3-1,tmax-1),64*c1+63),64*c2+62);c7++) {
          for (c9=64*c3;c9<=min(c7+nx-1,64*c3+63);c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c9=64*c3;c9<=min(c7+nx,64*c3+63);c9++) {
            hz[-c7+c9-1][0]=hz[-c7+c9-1][0]-((double)(7))/10*(ey[1+-c7+c9-1][0]+ex[-c7+c9-1][1+0]-ex[-c7+c9-1][0]-ey[-c7+c9-1][0]);
          }
        }
      }
      if ((c1 == c3) && (c2 <= min(floord(64*c3-1,64),floord(tmax-64,64))) && (nx >= 2) && (ny == 1)) {
        ey[0][0]=64*c2+63;
        for (c9=64*c2+64;c9<=min(64*c2+nx+62,64*c3+63);c9++) {
          ey[-64*c2+c9-63][0]=ey[-64*c2+c9-63][0]-((double)(1))/2*(hz[-64*c2+c9-63][0]-hz[-64*c2+c9-63 -1][0]);
        }
      }
      if ((c1 == c3) && (c2 <= min(floord(64*c3-1,64),floord(tmax-64,64))) && (nx >= 2) && (ny >= 2)) {
        ey[0][0]=64*c2+63;
        for (c9=64*c2+64;c9<=min(64*c2+nx+62,64*c3+63);c9++) {
          ey[-64*c2+c9-63][0]=ey[-64*c2+c9-63][0]-((double)(1))/2*(hz[-64*c2+c9-63][0]-hz[-64*c2+c9-63 -1][0]);
        }
      }
      if ((-c1 == -c3) && (c1 >= ceild(64*c2-63,64)) && (c1 <= min(floord(tmax-64,64),floord(64*c2-1,64))) && (nx == 1) && (ny >= 2)) {
        ey[0][0]=64*c1+63;
        for (c8=64*c1+64;c8<=min(64*c2+63,64*c1+ny+62);c8++) {
          ey[0][-64*c1+c8-63]=64*c1+63;
          ex[0][-64*c1+c8-63]=ex[0][-64*c1+c8-63]-((double)(1))/2*(hz[0][-64*c1+c8-63]-hz[0][-64*c1+c8-63 -1]);
        }
      }
      if ((-c1 == -c3) && (c1 <= min(floord(tmax-64,64),c2-1)) && (nx == 1)) {
        for (c8=64*c2;c8<=min(64*c2+63,64*c1+ny+62);c8++) {
          ey[0][-64*c1+c8-63]=64*c1+63;
          ex[0][-64*c1+c8-63]=ex[0][-64*c1+c8-63]-((double)(1))/2*(hz[0][-64*c1+c8-63]-hz[0][-64*c1+c8-63 -1]);
        }
      }
      if ((c1 == c3) && (c2 <= min(c3,floord(tmax-64,64))) && (nx == 1) && (ny == 1)) {
        ey[0][0]=64*c2+63;
      }
      if ((-c1 == -c3) && (c1 <= min(floord(tmax-64,64),floord(64*c2-1,64))) && (nx == 1) && (ny == 1)) {
        ey[0][0]=64*c1+63;
      }
      if ((c1 == c3) && (c2 <= min(c3,floord(tmax-64,64))) && (nx == 1) && (ny >= 2)) {
        ey[0][0]=64*c2+63;
      }
      if ((-c1 == -c3) && (c1 >= ceild(64*c2-63,64)) && (c1 <= min(floord(tmax-64,64),floord(64*c2-1,64))) && (nx >= 2) && (ny >= 2)) {
        ey[0][0]=64*c1+63;
        for (c8=64*c1+64;c8<=min(64*c2+63,64*c1+ny+62);c8++) {
          ey[0][-64*c1+c8-63]=64*c1+63;
          ex[0][-64*c1+c8-63]=ex[0][-64*c1+c8-63]-((double)(1))/2*(hz[0][-64*c1+c8-63]-hz[0][-64*c1+c8-63 -1]);
        }
      }
      if ((-c1 == -c3) && (c1 <= min(floord(tmax-64,64),c2-1)) && (nx >= 2)) {
        for (c8=64*c2;c8<=min(64*c2+63,64*c1+ny+62);c8++) {
          ey[0][-64*c1+c8-63]=64*c1+63;
          ex[0][-64*c1+c8-63]=ex[0][-64*c1+c8-63]-((double)(1))/2*(hz[0][-64*c1+c8-63]-hz[0][-64*c1+c8-63 -1]);
        }
      }
      if ((-c1 == -c3) && (c1 <= min(c2,floord(tmax-64,64))) && (nx >= 2) && (ny == 1)) {
        ey[0][0]=64*c1+63;
      }
      if ((-c1 == -c2) && (-c1 == -c3) && (c1 <= floord(tmax-64,64)) && (nx >= 2) && (ny >= 2)) {
        ey[0][0]=64*c1+63;
      }
      if ((c1 >= c2) && (c2 <= min(floord(tmax-64,64),c3-1)) && (ny == 1)) {
        for (c9=64*c3;c9<=min(64*c2+nx+62,64*c3+63);c9++) {
          ey[-64*c2+c9-63][0]=ey[-64*c2+c9-63][0]-((double)(1))/2*(hz[-64*c2+c9-63][0]-hz[-64*c2+c9-63 -1][0]);
        }
      }
      if ((c1 >= c2) && (c2 <= min(floord(tmax-64,64),c3-1)) && (ny >= 2)) {
        for (c9=64*c3;c9<=min(64*c2+nx+62,64*c3+63);c9++) {
          ey[-64*c2+c9-63][0]=ey[-64*c2+c9-63][0]-((double)(1))/2*(hz[-64*c2+c9-63][0]-hz[-64*c2+c9-63 -1][0]);
        }
      }
    }
  }
}
/* polysyn end */



/*@ end @*/
/*@ end @*/



    annot_t_end = rtclock();
    annot_t_total += annot_t_end - annot_t_start;
  }
  
  annot_t_total = annot_t_total / REPS;

#ifndef TEST
  printf("%f\n", annot_t_total);
#else
  {
    int i,j;
    for (i=0; i<nx; i++) {
      for (j=0; j<ny; j++)  {
	if (j%100==0)
          printf("\n");
        printf("%f ",hz[i][j]);
      }
      printf("\n");
    }
  }
#endif


  return ((int) hz[0][0]); 

}
                                    
