#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>

#define ceild(n,d)  ceil(((double)(n))/((double)(d)))
#define floord(n,d) floor(((double)(n))/((double)(d)))
#define max(x,y)    ((x) > (y)? (x) : (y))
#define min(x,y)    ((x) < (y)? (x) : (y))

double L[N][N];
double U[N][N];
double A[N][N+13];

void init_arrays()
{
  int i, j, k;

  /* have to initialize this matrix properly to prevent                                              
   * division by zero                                                                                 
   */
  for (i=0; i<N; i++) {
    for (j=0; j<N; j++) {
      L[i][j] = 0.0;
      U[i][j] = 0.0;
    }
  }

  for (i=0; i<N; i++) {
    for (j=0; j<=i; j++) {
      L[i][j] = i+j+1;
      U[j][i] = i+j+1;
    }
  }

  for (i=0; i<N; i++) {
    for (j=0; j<N; j++) {
      for (k=0; k<N; k++) {
	A[i][j] += L[i][k]*U[k][j];
      }
    }
  }
}

double rtclock()
{
  struct timezone tzp;
  struct timeval tp;
  int stat;
  gettimeofday (&tp, &tzp);
  return (tp.tv_sec + tp.tv_usec*1.0e-6);
}

int main()
{
  init_arrays();

  double annot_t_start=0, annot_t_end=0, annot_t_total=0;
  int annot_i;

  for (annot_i=0; annot_i<REPS; annot_i++)
    {
      annot_t_start = rtclock();



/*@ begin PerfTuning (         
  def build 
  { 
    arg build_command = 'icc -O3 -openmp -I/usr/local/icc/include -lm'; 
  } 
    
  def performance_counter          
  { 
    arg repetitions = 50; 
  }

  def performance_params 
  {
    param PERM[] = [
     ['i','j'],
#     ['j','i'],
    ];

    param U1[] = [1];
    param U2[] = [1];
    param U3[] = [1];
    param U4[] = [24,26,28];

    param IVEC1[] = [False];
    param IVEC2[] = [False];
    param SCREP[] = [False];
  }

  def search 
  { 
    arg algorithm = 'Exhaustive'; 
#    arg algorithm = 'Simplex'; 
#    arg total_runs = 1;
  } 
   
  def input_params 
  {
    param N[] = [500];
  }

  def input_vars
  {
    arg decl_file = 'decl_code.h';
    arg init_file = 'init_code.c';
  }
) @*/

/**-- (Generated by Orio) 
Best performance cost: 
  0.031449 
Tuned for specific problem sizes: 
  N = 500 
Best performance parameters: 
  IVEC1 = False 
  IVEC2 = False 
  PERM = ['i', 'j'] 
  SCREP = False 
  U1 = 1 
  U2 = 1 
  U3 = 1 
  U4 = 24 
--**/

 

register int i,j,k,it,jt,kt;

/*@ begin Loop (
transform Unroll(ufactor=U4)
for (k=0; k<=N-1; k++) {

  transform Composite (
  scalarreplace = (SCREP, 'double'),
  regtile = (['j'],[U1]),
  vector = (IVEC1, ['ivdep','vector always']))
    for (j=k+1; j<=N-1; j++)
      A[k][j] = A[k][j]/A[k][k];

  transform Composite (
  permut = [PERM],
  scalarreplace = (SCREP, 'double'),
  regtile = (['i','j'],[U2,U3]),
  vector = (IVEC2, ['ivdep','vector always']))
    for(i=k+1; i<=N-1; i++)
      for (j=k+1; j<=N-1; j++)
        A[i][j] = A[i][j] - A[i][k]*A[k][j];
  }
) @*/{
  for (k=0; k<=N-24; k=k+24) {
    for (j=k+1; j<=N-1; j++ ) 
      A[k][j]=A[k][j]/A[k][k];
    for (i=k+1; i<=N-1; i++ ) 
      for (j=k+1; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][k]*A[k][j];
    for (j=k+2; j<=N-1; j++ ) 
      A[(k+1)][j]=A[(k+1)][j]/A[(k+1)][(k+1)];
    for (i=k+2; i<=N-1; i++ ) 
      for (j=k+2; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+1)]*A[(k+1)][j];
    for (j=k+3; j<=N-1; j++ ) 
      A[(k+2)][j]=A[(k+2)][j]/A[(k+2)][(k+2)];
    for (i=k+3; i<=N-1; i++ ) 
      for (j=k+3; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+2)]*A[(k+2)][j];
    for (j=k+4; j<=N-1; j++ ) 
      A[(k+3)][j]=A[(k+3)][j]/A[(k+3)][(k+3)];
    for (i=k+4; i<=N-1; i++ ) 
      for (j=k+4; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+3)]*A[(k+3)][j];
    for (j=k+5; j<=N-1; j++ ) 
      A[(k+4)][j]=A[(k+4)][j]/A[(k+4)][(k+4)];
    for (i=k+5; i<=N-1; i++ ) 
      for (j=k+5; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+4)]*A[(k+4)][j];
    for (j=k+6; j<=N-1; j++ ) 
      A[(k+5)][j]=A[(k+5)][j]/A[(k+5)][(k+5)];
    for (i=k+6; i<=N-1; i++ ) 
      for (j=k+6; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+5)]*A[(k+5)][j];
    for (j=k+7; j<=N-1; j++ ) 
      A[(k+6)][j]=A[(k+6)][j]/A[(k+6)][(k+6)];
    for (i=k+7; i<=N-1; i++ ) 
      for (j=k+7; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+6)]*A[(k+6)][j];
    for (j=k+8; j<=N-1; j++ ) 
      A[(k+7)][j]=A[(k+7)][j]/A[(k+7)][(k+7)];
    for (i=k+8; i<=N-1; i++ ) 
      for (j=k+8; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+7)]*A[(k+7)][j];
    for (j=k+9; j<=N-1; j++ ) 
      A[(k+8)][j]=A[(k+8)][j]/A[(k+8)][(k+8)];
    for (i=k+9; i<=N-1; i++ ) 
      for (j=k+9; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+8)]*A[(k+8)][j];
    for (j=k+10; j<=N-1; j++ ) 
      A[(k+9)][j]=A[(k+9)][j]/A[(k+9)][(k+9)];
    for (i=k+10; i<=N-1; i++ ) 
      for (j=k+10; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+9)]*A[(k+9)][j];
    for (j=k+11; j<=N-1; j++ ) 
      A[(k+10)][j]=A[(k+10)][j]/A[(k+10)][(k+10)];
    for (i=k+11; i<=N-1; i++ ) 
      for (j=k+11; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+10)]*A[(k+10)][j];
    for (j=k+12; j<=N-1; j++ ) 
      A[(k+11)][j]=A[(k+11)][j]/A[(k+11)][(k+11)];
    for (i=k+12; i<=N-1; i++ ) 
      for (j=k+12; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+11)]*A[(k+11)][j];
    for (j=k+13; j<=N-1; j++ ) 
      A[(k+12)][j]=A[(k+12)][j]/A[(k+12)][(k+12)];
    for (i=k+13; i<=N-1; i++ ) 
      for (j=k+13; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+12)]*A[(k+12)][j];
    for (j=k+14; j<=N-1; j++ ) 
      A[(k+13)][j]=A[(k+13)][j]/A[(k+13)][(k+13)];
    for (i=k+14; i<=N-1; i++ ) 
      for (j=k+14; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+13)]*A[(k+13)][j];
    for (j=k+15; j<=N-1; j++ ) 
      A[(k+14)][j]=A[(k+14)][j]/A[(k+14)][(k+14)];
    for (i=k+15; i<=N-1; i++ ) 
      for (j=k+15; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+14)]*A[(k+14)][j];
    for (j=k+16; j<=N-1; j++ ) 
      A[(k+15)][j]=A[(k+15)][j]/A[(k+15)][(k+15)];
    for (i=k+16; i<=N-1; i++ ) 
      for (j=k+16; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+15)]*A[(k+15)][j];
    for (j=k+17; j<=N-1; j++ ) 
      A[(k+16)][j]=A[(k+16)][j]/A[(k+16)][(k+16)];
    for (i=k+17; i<=N-1; i++ ) 
      for (j=k+17; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+16)]*A[(k+16)][j];
    for (j=k+18; j<=N-1; j++ ) 
      A[(k+17)][j]=A[(k+17)][j]/A[(k+17)][(k+17)];
    for (i=k+18; i<=N-1; i++ ) 
      for (j=k+18; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+17)]*A[(k+17)][j];
    for (j=k+19; j<=N-1; j++ ) 
      A[(k+18)][j]=A[(k+18)][j]/A[(k+18)][(k+18)];
    for (i=k+19; i<=N-1; i++ ) 
      for (j=k+19; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+18)]*A[(k+18)][j];
    for (j=k+20; j<=N-1; j++ ) 
      A[(k+19)][j]=A[(k+19)][j]/A[(k+19)][(k+19)];
    for (i=k+20; i<=N-1; i++ ) 
      for (j=k+20; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+19)]*A[(k+19)][j];
    for (j=k+21; j<=N-1; j++ ) 
      A[(k+20)][j]=A[(k+20)][j]/A[(k+20)][(k+20)];
    for (i=k+21; i<=N-1; i++ ) 
      for (j=k+21; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+20)]*A[(k+20)][j];
    for (j=k+22; j<=N-1; j++ ) 
      A[(k+21)][j]=A[(k+21)][j]/A[(k+21)][(k+21)];
    for (i=k+22; i<=N-1; i++ ) 
      for (j=k+22; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+21)]*A[(k+21)][j];
    for (j=k+23; j<=N-1; j++ ) 
      A[(k+22)][j]=A[(k+22)][j]/A[(k+22)][(k+22)];
    for (i=k+23; i<=N-1; i++ ) 
      for (j=k+23; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+22)]*A[(k+22)][j];
    for (j=k+24; j<=N-1; j++ ) 
      A[(k+23)][j]=A[(k+23)][j]/A[(k+23)][(k+23)];
    for (i=k+24; i<=N-1; i++ ) 
      for (j=k+24; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][(k+23)]*A[(k+23)][j];
  }
  for (; k<=N-1; k=k+1) {
    for (j=k+1; j<=N-1; j++ ) 
      A[k][j]=A[k][j]/A[k][k];
    for (i=k+1; i<=N-1; i++ ) 
      for (j=k+1; j<=N-1; j++ ) 
        A[i][j]=A[i][j]-A[i][k]*A[k][j];
  }
}
/*@ end @*/

/*@ end @*/





      annot_t_end = rtclock();
      annot_t_total += annot_t_end - annot_t_start;
    }

  annot_t_total = annot_t_total / REPS;

#ifndef TEST
  printf("%f\n", annot_t_total);
#else
  {
    int i, j;
    for (i=0; i<N; i++) {
      for (j=0; j<N; j++) {
        if (j%100==0)
          printf("\n");
        printf("%f ",A[i][j]);
      }
      printf("\n");
    }
  }
#endif

  return ((int) A[0][0]);

}
