
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>

#define tmax T
#define nx N
#define ny N
double ex[nx][ny +1];
double ey[nx +1][ny];
double hz[nx][ny];

void init_arrays()
{
    int i, j;
    for (i=0; i<nx+1; i++)  {
        for (j=0; j<ny; j++)  {
            ey[i][j] = 0;
        }
    }
    for (i=0; i<nx; i++)  {
        for (j=0; j<ny+1; j++)  {
            ex[i][j] = 0;
        }
    }
    for (j=0; j<ny; j++)  {
        ey[0][j] = ((double)j)/ny;
    }
    for (i=0; i<nx; i++)    {
        for (j=0; j<ny; j++)  {
            hz[i][j] = 0;
        }
    }
}

double rtclock()
{
  struct timezone tzp;
  struct timeval tp;
  int stat;
  gettimeofday (&tp, &tzp);
  return (tp.tv_sec + tp.tv_usec*1.0e-6);
}

int main()
{
  init_arrays();

  double annot_t_start=0, annot_t_end=0, annot_t_total=0;
  int annot_i;

  for (annot_i=0; annot_i<REPS; annot_i++)
  {
    annot_t_start = rtclock();
    
/*@ begin PerfTuning (
  def build {
    arg build_command = 'icc -O3 -openmp -lm';
  }

  def performance_counter {
    arg repetitions = 1;
  }  
  
  def performance_params {  
#   [4,8,16,32,64,128];
#   [1,4,8,16];

    param T1_1[] = [16];
    param T1_2[] = [32];
    param T1_3[] = [32];
    param T2_1[] = [1];
    param T2_2[] = [1];
    param T2_3[] = [1];

#    constraint c1 = (T1_1*T2_1<=1024 and T1_2*T2_2<=1024 and T1_3*T2_3<=1024);
#    constraint c2 = (T1_1==T1_3 and T2_1==T2_3);
    
    param U1[] = [8];
    param U2[] = [2];
    param U3[] = [4];

    constraint c3 = (U1*U2*U3<=256);

    param PERM[] = [
#      [0,1,2],
      [0,2,1],
#      [1,0,2],
#      [1,2,0],
#      [2,0,1],
#      [2,1,0],

    ];

    param PAR[] = [True];
    param SCREP[] = [False];
    param IVEC[] = [True];
  }
  
  def search
  {
   arg algorithm = 'Exhaustive';
#   arg algorithm = 'Simplex';
#   arg total_runs = 1;
#   arg algorithm = 'Random';
#   arg time_limit = 10;
  }

  def input_params
  {
  let N=2000;
  param tmax[] = [500];
  param nx[] = [N];
  param ny[] = [N]; 
  }

  def input_vars
  {
  decl static double ex[nx][ny+1] = random;
  decl static double ey[nx+1][ny] = random;
  decl static double hz[nx][ny] = random;
  }
) @*/

/**-- (Generated by Orio) 
Best performance cost: 
  1.681530 
Tuned for specific problem sizes: 
  nx = 2000 
  ny = 2000 
  tmax = 500 
Best performance parameters: 
  IVEC = True 
  PAR = True 
  PERM = [0, 2, 1] 
  SCREP = False 
  T1_1 = 16 
  T1_2 = 32 
  T1_3 = 32 
  T2_1 = 1 
  T2_2 = 1 
  T2_3 = 1 
  U1 = 8 
  U2 = 2 
  U3 = 4 
--**/

   

register int i,j,k,t;  
register int c1t, c2t, c3t, c4t, c5t, c6t, c7t, c8t, c9t, c10t, c11t, c12t;  
register int newlb_c1, newlb_c2, newlb_c3, newlb_c4, newlb_c5, newlb_c6,  
  newlb_c7, newlb_c8, newlb_c9, newlb_c10, newlb_c11, newlb_c12;  
register int newub_c1, newub_c2, newub_c3, newub_c4, newub_c5, newub_c6,  
  newub_c7, newub_c8, newub_c9, newub_c10, newub_c11, newub_c12;  

/*@ begin PolySyn(  
  parallel = PAR; 
  tiles = [T1_1,T1_2,T1_3,T2_1,T2_2,T2_3];
  permut = PERM;
  unroll_factors = [U1,U2,U3]; 
  scalar_replace = SCREP; 
  vectorize = IVEC; 
 
  profiling_code = 'fdtd-2d_profiling.c'; 
  compile_cmd = 'gcc'; 
  compile_opts = '-lm'; 
  ) @*/#include <math.h>
#include <assert.h>
#include <omp.h>
#define ceild(n,d)  ceil(((double)(n))/((double)(d)))
#define floord(n,d) floor(((double)(n))/((double)(d)))
#define max(x,y)    ((x) > (y)? (x) : (y))
#define min(x,y)    ((x) < (y)? (x) : (y))
 








 int c1, c2, c3, c4, c5, c6, c7, c8, c9;
 register int lb, ub, lb1, ub1, lb2, ub2;
/* polysyn start */
for (c1=-1;c1<=floord(3*tmax+ny-3,32);c1++) {
 lb1=max(max(ceild(16*c1-tmax+1,16),0),ceild(16*c1-31,48));
 ub1=min(min(floord(16*c1+15,16),floord(16*c1+ny+15,48)),floord(tmax+ny-1,32));
#pragma omp parallel for shared(c1,lb1,ub1) private(c2,c3,c4,c5,c6,c7,c8,c9)
 for (c2=lb1; c2<=ub1; c2++) {
    for (c3=max(max(max(max(max(max(max(max(max(max(ceild(-ny-29,32),ceild(-32*c2-61,32)),ceild(32*c2-ny-30,32)),0),ceild(32*c1-64*c2-61,32)),ceild(16*c1-16*c2-31,32)),ceild(512*c1-1504*c2-30*nx-931,32)),ceild(496*c1-1456*c2-30*nx-ny-899,32)),ceild(16*c1-16*c2-ny-29,32)),ceild(32*c1-992*c2-30*nx-ny-899,32)),ceild(32*c1-1024*c2-30*nx-931,32));c3<=min(min(min(floord(16*c1-16*c2+nx+15,32),floord(32*c1+960*c2+31*nx+930,32)),floord(32*c2+nx+30,32)),floord(tmax+nx-1,32));c3++) {
      if ((c1 <= floord(16*c2+32*c3-nx,16)) && (c2 <= floord(32*c3-nx+ny,32)) && (c3 >= ceild(nx,32))) {
        for (c8=max(32*c3-nx+1,32*c2);c8<=min(32*c3-nx+ny,32*c2+31);c8++) {
          hz[nx-1][-32*c3+c8+nx-1]=hz[nx-1][-32*c3+c8+nx-1]-((double)(7))/10*(ey[1+nx-1][-32*c3+c8+nx-1]+ex[nx-1][1+-32*c3+c8+nx-1]-ex[nx-1][-32*c3+c8+nx-1]-ey[nx-1][-32*c3+c8+nx-1]);
        }
      }
      if ((c1 <= floord(48*c2-ny,16)) && (c2 >= max(ceild(ny,32),ceild(32*c3-nx+ny+1,32))) && (nx >= 2)) {
        for (c9=max(32*c3,32*c2-ny+1);c9<=min(32*c3+31,32*c2+nx-ny);c9++) {
          hz[-32*c2+c9+ny-1][ny-1]=hz[-32*c2+c9+ny-1][ny-1]-((double)(7))/10*(ey[1+-32*c2+c9+ny-1][ny-1]+ex[-32*c2+c9+ny-1][1+ny-1]-ex[-32*c2+c9+ny-1][ny-1]-ey[-32*c2+c9+ny-1][ny-1]);
        }
      }
      if ((c1 <= floord(48*c2-ny,16)) && (c2 >= max(ceild(ny,32),ceild(32*c3+ny,32))) && (nx == 1) && (ny >= 2)) {
        hz[0][ny-1]=hz[0][ny-1]-((double)(7))/10*(ey[1+0][ny-1]+ex[0][1+ny-1]-ex[0][ny-1]-ey[0][ny-1]);
      }
      if ((c1 <= floord(48*c2-1,16)) && (c2 <= floord(c3+1,2)) && (c2 >= max(ceild(32*c3+1,32),1)) && (nx == 1) && (ny == 1)) {
        hz[0][0]=hz[0][0]-((double)(7))/10*(ey[1+0][0]+ex[0][1+0]-ex[0][0]-ey[0][0]);
      }
      if ((c1 == c2+c3) && (nx == 1)) {
        for (c7=max(max(32*c3,0),32*c2-ny+1);c7<=min(min(min(32*c2-1,tmax-1),16*c3+15),32*c2-ny+31);c7++) {
          for (c8=32*c2;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
          hz[0][ny-1]=hz[0][ny-1]-((double)(7))/10*(ey[1+0][ny-1]+ex[0][1+ny-1]-ex[0][ny-1]-ey[0][ny-1]);
        }
      }
      if ((c1 == c2+c3) && (nx == 1) && (ny >= 2)) {
        for (c7=max(max(32*c2,32*c3),0);c7<=min(min(tmax-1,16*c3+15),32*c2-ny+31);c7++) {
          ey[0][0]=c7;
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
          hz[0][ny-1]=hz[0][ny-1]-((double)(7))/10*(ey[1+0][ny-1]+ex[0][1+ny-1]-ex[0][ny-1]-ey[0][ny-1]);
        }
      }
      if ((c1 == c2+c3) && (nx == 1)) {
        for (c7=max(max(32*c3,0),32*c2-ny+32);c7<=min(min(tmax-1,16*c3+15),32*c2-1);c7++) {
          for (c8=32*c2;c8<=32*c2+31;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
        }
      }
      if ((c1 == c2+c3) && (nx == 1)) {
        for (c7=max(max(max(32*c2,32*c3),0),32*c2-ny+32);c7<=min(min(tmax-1,16*c3+15),32*c2+30);c7++) {
          ey[0][0]=c7;
          for (c8=c7+1;c8<=32*c2+31;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
        }
      }
      if ((c1 == c2+c3) && (nx == 1) && (ny == 1)) {
        for (c7=max(max(32*c3,0),32*c2);c7<=min(min(tmax-1,16*c3+15),32*c2+30);c7++) {
          ey[0][0]=c7;
          hz[0][0]=hz[0][0]-((double)(7))/10*(ey[1+0][0]+ex[0][1+0]-ex[0][0]-ey[0][0]);
        }
      }
      for (c7=max(max(max(0,32*c3-nx+1),32*c2-ny+1),16*c1-16*c2);c7<=min(min(min(min(min(32*c2-1,32*c3-nx+31),32*c3-1),tmax-1),32*c2-ny+31),16*c1-16*c2+15);c7++) {
        for (c8=32*c2;c8<=c7+ny-1;c8++) {
          for (c9=32*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
          hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
        }
        for (c9=32*c3;c9<=c7+nx;c9++) {
          hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
        }
      }
      if (ny >= 2) {
        for (c7=max(max(max(32*c2,0),32*c3-nx+1),16*c1-16*c2);c7<=min(min(min(min(32*c3-nx+31,32*c3-1),tmax-1),32*c2-ny+31),16*c1-16*c2+15);c7++) {
          for (c9=32*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            for (c9=32*c3;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=32*c3;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      for (c7=max(max(max(0,16*c1-16*c2),32*c3-nx+1),32*c2-ny+32);c7<=min(min(min(min(tmax-1,32*c3-1),16*c1-16*c2+15),32*c2-1),32*c3-nx+31);c7++) {
        for (c8=32*c2;c8<=32*c2+31;c8++) {
          for (c9=32*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
          hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
        }
      }
      for (c7=max(max(max(32*c3-nx+32,0),32*c2-ny+1),16*c1-16*c2);c7<=min(min(min(min(32*c2-1,32*c3-1),tmax-1),32*c2-ny+31),16*c1-16*c2+15);c7++) {
        for (c8=32*c2;c8<=c7+ny-1;c8++) {
          for (c9=32*c3;c9<=32*c3+31;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
        for (c9=32*c3;c9<=32*c3+31;c9++) {
          hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
        }
      }
      for (c7=max(max(max(max(32*c2,0),16*c1-16*c2),32*c3-nx+1),32*c2-ny+32);c7<=min(min(min(min(tmax-1,32*c3-1),32*c2+30),16*c1-16*c2+15),32*c3-nx+31);c7++) {
        for (c9=32*c3;c9<=c7+nx-1;c9++) {
          ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
        }
        for (c8=c7+1;c8<=32*c2+31;c8++) {
          for (c9=32*c3;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
          hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
        }
      }
      if (ny >= 2) {
        for (c7=max(max(max(32*c2,32*c3-nx+32),0),16*c1-16*c2);c7<=min(min(min(32*c3-1,tmax-1),32*c2-ny+31),16*c1-16*c2+15);c7++) {
          for (c9=32*c3;c9<=32*c3+31;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            for (c9=32*c3;c9<=32*c3+31;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=32*c3;c9<=32*c3+31;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
/*@ begin Loop(
transform Composite(
permut = [['c7', 'c9', 'c8']],
  regtile = (['c7', 'c8', 'c9'],[8, 2, 4]),
  scalarreplace = (False, 'double'),
  vector = (True, ['ivdep','vector always']))

      for (c7=max(max(max(0,16*c1-16*c2),32*c3-nx+32),32*c2-ny+32);c7<=min(min(min(tmax-1,32*c3-1),16*c1-16*c2+15),32*c2-1);c7++) {
        for (c8=32*c2;c8<=32*c2+31;c8++) {
          for (c9=32*c3;c9<=32*c3+31;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
      }

) @*/{
  for (c7t=max(max(max(0,16*c1-16*c2),32*c3-nx+32),32*c2-ny+32); c7t<=min(min(min(tmax-1,32*c3-1),16*c1-16*c2+15),32*c2-1)-7; c7t=c7t+8) {
    for (c9t=32*c3; c9t<=32*c3+28; c9t=c9t+4) {
      register int cbv_1, cbv_2;
      cbv_1=32*c2;
      cbv_2=32*c2+30;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_1; c8t<=cbv_2; c8t=c8t+2) {
        ey[-c7t+c9t][-c7t+c8t]=ey[-c7t+c9t][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8t]-hz[-c7t+c9t-1][-c7t+c8t]);
        ey[-c7t+c9t][-c7t+(c8t+1)]=ey[-c7t+c9t][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9t][-c7t+(c8t+1)]-hz[-c7t+c9t-1][-c7t+(c8t+1)]);
        ey[-c7t+(c9t+1)][-c7t+c8t]=ey[-c7t+(c9t+1)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8t]-hz[-c7t+(c9t+1)-1][-c7t+c8t]);
        ey[-c7t+(c9t+1)][-c7t+(c8t+1)]=ey[-c7t+(c9t+1)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+(c8t+1)]-hz[-c7t+(c9t+1)-1][-c7t+(c8t+1)]);
        ey[-c7t+(c9t+2)][-c7t+c8t]=ey[-c7t+(c9t+2)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8t]-hz[-c7t+(c9t+2)-1][-c7t+c8t]);
        ey[-c7t+(c9t+2)][-c7t+(c8t+1)]=ey[-c7t+(c9t+2)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+(c8t+1)]-hz[-c7t+(c9t+2)-1][-c7t+(c8t+1)]);
        ey[-c7t+(c9t+3)][-c7t+c8t]=ey[-c7t+(c9t+3)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+3)][-c7t+c8t]-hz[-c7t+(c9t+3)-1][-c7t+c8t]);
        ey[-c7t+(c9t+3)][-c7t+(c8t+1)]=ey[-c7t+(c9t+3)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+3)][-c7t+(c8t+1)]-hz[-c7t+(c9t+3)-1][-c7t+(c8t+1)]);
        ey[-(c7t+1)+c9t][-(c7t+1)+c8t]=ey[-(c7t+1)+c9t][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8t]-hz[-(c7t+1)+c9t-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]=ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]=ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t]=ey[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+2)+c9t][-(c7t+2)+c8t]=ey[-(c7t+2)+c9t][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8t]-hz[-(c7t+2)+c9t-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]=ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]=ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t]=ey[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+3)+c9t][-(c7t+3)+c8t]=ey[-(c7t+3)+c9t][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8t]-hz[-(c7t+3)+c9t-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]=ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]=ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t]=ey[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+4)+c9t][-(c7t+4)+c8t]=ey[-(c7t+4)+c9t][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8t]-hz[-(c7t+4)+c9t-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]=ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]=ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t]=ey[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+5)+c9t][-(c7t+5)+c8t]=ey[-(c7t+5)+c9t][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8t]-hz[-(c7t+5)+c9t-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]=ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]=ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t]=ey[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+6)+c9t][-(c7t+6)+c8t]=ey[-(c7t+6)+c9t][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8t]-hz[-(c7t+6)+c9t-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]=ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]=ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t]=ey[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+7)+c9t][-(c7t+7)+c8t]=ey[-(c7t+7)+c9t][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+c9t][-(c7t+7)+c8t]-hz[-(c7t+7)+c9t-1][-(c7t+7)+c8t]);
        ey[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)]=ey[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+c9t-1][-(c7t+7)+(c8t+1)]);
        ey[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t]=ey[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t]-hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8t]);
        ey[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)]=ey[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+(c8t+1)]);
        ey[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t]=ey[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t]-hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8t]);
        ey[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)]=ey[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+(c8t+1)]);
        ey[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t]=ey[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t]-hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8t]);
        ey[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)]=ey[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+(c8t+1)]);
        ex[-c7t+c9t][-c7t+c8t]=ex[-c7t+c9t][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8t]-hz[-c7t+c9t][-c7t+c8t-1]);
        ex[-c7t+c9t][-c7t+(c8t+1)]=ex[-c7t+c9t][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9t][-c7t+(c8t+1)]-hz[-c7t+c9t][-c7t+(c8t+1)-1]);
        ex[-c7t+(c9t+1)][-c7t+c8t]=ex[-c7t+(c9t+1)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8t]-hz[-c7t+(c9t+1)][-c7t+c8t-1]);
        ex[-c7t+(c9t+1)][-c7t+(c8t+1)]=ex[-c7t+(c9t+1)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+(c8t+1)]-hz[-c7t+(c9t+1)][-c7t+(c8t+1)-1]);
        ex[-c7t+(c9t+2)][-c7t+c8t]=ex[-c7t+(c9t+2)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8t]-hz[-c7t+(c9t+2)][-c7t+c8t-1]);
        ex[-c7t+(c9t+2)][-c7t+(c8t+1)]=ex[-c7t+(c9t+2)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+(c8t+1)]-hz[-c7t+(c9t+2)][-c7t+(c8t+1)-1]);
        ex[-c7t+(c9t+3)][-c7t+c8t]=ex[-c7t+(c9t+3)][-c7t+c8t]-((double)(1))/2*(hz[-c7t+(c9t+3)][-c7t+c8t]-hz[-c7t+(c9t+3)][-c7t+c8t-1]);
        ex[-c7t+(c9t+3)][-c7t+(c8t+1)]=ex[-c7t+(c9t+3)][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+(c9t+3)][-c7t+(c8t+1)]-hz[-c7t+(c9t+3)][-c7t+(c8t+1)-1]);
        ex[-(c7t+1)+c9t][-(c7t+1)+c8t]=ex[-(c7t+1)+c9t][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8t]-hz[-(c7t+1)+c9t][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9t][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]=ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+1)][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]=ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+2)][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t]=ex[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t]-hz[-(c7t+1)+(c9t+3)][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+(c9t+3)][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+2)+c9t][-(c7t+2)+c8t]=ex[-(c7t+2)+c9t][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8t]-hz[-(c7t+2)+c9t][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9t][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]=ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+1)][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]=ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+2)][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t]=ex[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t]-hz[-(c7t+2)+(c9t+3)][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+(c9t+3)][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+3)+c9t][-(c7t+3)+c8t]=ex[-(c7t+3)+c9t][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8t]-hz[-(c7t+3)+c9t][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9t][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]=ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+1)][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]=ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+2)][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t]=ex[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t]-hz[-(c7t+3)+(c9t+3)][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+(c9t+3)][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+4)+c9t][-(c7t+4)+c8t]=ex[-(c7t+4)+c9t][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8t]-hz[-(c7t+4)+c9t][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9t][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]=ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+1)][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]=ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+2)][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t]=ex[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t]-hz[-(c7t+4)+(c9t+3)][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+(c9t+3)][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+5)+c9t][-(c7t+5)+c8t]=ex[-(c7t+5)+c9t][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8t]-hz[-(c7t+5)+c9t][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9t][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]=ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+1)][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]=ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+2)][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t]=ex[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t]-hz[-(c7t+5)+(c9t+3)][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+(c9t+3)][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+6)+c9t][-(c7t+6)+c8t]=ex[-(c7t+6)+c9t][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8t]-hz[-(c7t+6)+c9t][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9t][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]=ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+1)][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]=ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+2)][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t]=ex[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t]-hz[-(c7t+6)+(c9t+3)][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+(c9t+3)][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+7)+c9t][-(c7t+7)+c8t]=ex[-(c7t+7)+c9t][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+c9t][-(c7t+7)+c8t]-hz[-(c7t+7)+c9t][-(c7t+7)+c8t-1]);
        ex[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)]=ex[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+c9t][-(c7t+7)+(c8t+1)-1]);
        ex[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t]=ex[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t]-hz[-(c7t+7)+(c9t+1)][-(c7t+7)+c8t-1]);
        ex[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)]=ex[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+(c9t+1)][-(c7t+7)+(c8t+1)-1]);
        ex[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t]=ex[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t]-hz[-(c7t+7)+(c9t+2)][-(c7t+7)+c8t-1]);
        ex[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)]=ex[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+(c9t+2)][-(c7t+7)+(c8t+1)-1]);
        ex[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t]=ex[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t]-hz[-(c7t+7)+(c9t+3)][-(c7t+7)+c8t-1]);
        ex[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)]=ex[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+(c9t+3)][-(c7t+7)+(c8t+1)-1]);
        hz[-c7t+c9t-1][-c7t+c8t-1]=hz[-c7t+c9t-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+c9t-1][-c7t+c8t-1]+ex[-c7t+c9t-1][1+-c7t+c8t-1]-ex[-c7t+c9t-1][-c7t+c8t-1]-ey[-c7t+c9t-1][-c7t+c8t-1]);
        hz[-c7t+c9t-1][-c7t+(c8t+1)-1]=hz[-c7t+c9t-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+c9t-1][-c7t+(c8t+1)-1]+ex[-c7t+c9t-1][1+-c7t+(c8t+1)-1]-ex[-c7t+c9t-1][-c7t+(c8t+1)-1]-ey[-c7t+c9t-1][-c7t+(c8t+1)-1]);
        hz[-c7t+(c9t+1)-1][-c7t+c8t-1]=hz[-c7t+(c9t+1)-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+(c9t+1)-1][-c7t+c8t-1]+ex[-c7t+(c9t+1)-1][1+-c7t+c8t-1]-ex[-c7t+(c9t+1)-1][-c7t+c8t-1]-ey[-c7t+(c9t+1)-1][-c7t+c8t-1]);
        hz[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]=hz[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]+ex[-c7t+(c9t+1)-1][1+-c7t+(c8t+1)-1]-ex[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]-ey[-c7t+(c9t+1)-1][-c7t+(c8t+1)-1]);
        hz[-c7t+(c9t+2)-1][-c7t+c8t-1]=hz[-c7t+(c9t+2)-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+(c9t+2)-1][-c7t+c8t-1]+ex[-c7t+(c9t+2)-1][1+-c7t+c8t-1]-ex[-c7t+(c9t+2)-1][-c7t+c8t-1]-ey[-c7t+(c9t+2)-1][-c7t+c8t-1]);
        hz[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]=hz[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]+ex[-c7t+(c9t+2)-1][1+-c7t+(c8t+1)-1]-ex[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]-ey[-c7t+(c9t+2)-1][-c7t+(c8t+1)-1]);
        hz[-c7t+(c9t+3)-1][-c7t+c8t-1]=hz[-c7t+(c9t+3)-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+(c9t+3)-1][-c7t+c8t-1]+ex[-c7t+(c9t+3)-1][1+-c7t+c8t-1]-ex[-c7t+(c9t+3)-1][-c7t+c8t-1]-ey[-c7t+(c9t+3)-1][-c7t+c8t-1]);
        hz[-c7t+(c9t+3)-1][-c7t+(c8t+1)-1]=hz[-c7t+(c9t+3)-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+(c9t+3)-1][-c7t+(c8t+1)-1]+ex[-c7t+(c9t+3)-1][1+-c7t+(c8t+1)-1]-ex[-c7t+(c9t+3)-1][-c7t+(c8t+1)-1]-ey[-c7t+(c9t+3)-1][-c7t+(c8t+1)-1]);
        hz[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+c9t-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+c9t-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+c9t-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+c9t-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+(c9t+1)-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+(c9t+1)-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+(c9t+1)-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+(c9t+2)-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+(c9t+2)-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+(c9t+2)-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+(c9t+3)-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+3)-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+(c9t+3)-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+(c9t+3)-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+(c9t+3)-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+c9t-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+c9t-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+c9t-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+c9t-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+(c9t+1)-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+(c9t+1)-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+(c9t+1)-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+(c9t+2)-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+(c9t+2)-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+(c9t+2)-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+(c9t+3)-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+3)-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+(c9t+3)-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+(c9t+3)-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+(c9t+3)-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+c9t-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+c9t-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+c9t-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+c9t-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+(c9t+1)-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+(c9t+1)-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+(c9t+1)-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+(c9t+2)-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+(c9t+2)-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+(c9t+2)-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+(c9t+3)-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+3)-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+(c9t+3)-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+(c9t+3)-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+(c9t+3)-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+c9t-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+c9t-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+c9t-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+c9t-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+(c9t+1)-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+(c9t+1)-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+(c9t+1)-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+(c9t+2)-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+(c9t+2)-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+(c9t+2)-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+(c9t+3)-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+3)-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+(c9t+3)-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+(c9t+3)-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+(c9t+3)-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+c9t-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+c9t-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+c9t-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+c9t-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+(c9t+1)-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+(c9t+1)-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+(c9t+1)-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+(c9t+2)-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+(c9t+2)-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+(c9t+2)-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+(c9t+3)-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+3)-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+(c9t+3)-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+(c9t+3)-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+(c9t+3)-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+c9t-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+c9t-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+c9t-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+c9t-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+(c9t+1)-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+(c9t+1)-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+(c9t+1)-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+(c9t+2)-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+(c9t+2)-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+(c9t+2)-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+(c9t+3)-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+3)-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+(c9t+3)-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+(c9t+3)-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+(c9t+3)-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+7)+c9t-1][-(c7t+7)+c8t-1]=hz[-(c7t+7)+c9t-1][-(c7t+7)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+7)+c9t-1][-(c7t+7)+c8t-1]+ex[-(c7t+7)+c9t-1][1+-(c7t+7)+c8t-1]-ex[-(c7t+7)+c9t-1][-(c7t+7)+c8t-1]-ey[-(c7t+7)+c9t-1][-(c7t+7)+c8t-1]);
        hz[-(c7t+7)+c9t-1][-(c7t+7)+(c8t+1)-1]=hz[-(c7t+7)+c9t-1][-(c7t+7)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+7)+c9t-1][-(c7t+7)+(c8t+1)-1]+ex[-(c7t+7)+c9t-1][1+-(c7t+7)+(c8t+1)-1]-ex[-(c7t+7)+c9t-1][-(c7t+7)+(c8t+1)-1]-ey[-(c7t+7)+c9t-1][-(c7t+7)+(c8t+1)-1]);
        hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8t-1]=hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8t-1]+ex[-(c7t+7)+(c9t+1)-1][1+-(c7t+7)+c8t-1]-ex[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8t-1]-ey[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8t-1]);
        hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+(c8t+1)-1]=hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+1)-1][-(c7t+7)+(c8t+1)-1]+ex[-(c7t+7)+(c9t+1)-1][1+-(c7t+7)+(c8t+1)-1]-ex[-(c7t+7)+(c9t+1)-1][-(c7t+7)+(c8t+1)-1]-ey[-(c7t+7)+(c9t+1)-1][-(c7t+7)+(c8t+1)-1]);
        hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8t-1]=hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8t-1]+ex[-(c7t+7)+(c9t+2)-1][1+-(c7t+7)+c8t-1]-ex[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8t-1]-ey[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8t-1]);
        hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+(c8t+1)-1]=hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+2)-1][-(c7t+7)+(c8t+1)-1]+ex[-(c7t+7)+(c9t+2)-1][1+-(c7t+7)+(c8t+1)-1]-ex[-(c7t+7)+(c9t+2)-1][-(c7t+7)+(c8t+1)-1]-ey[-(c7t+7)+(c9t+2)-1][-(c7t+7)+(c8t+1)-1]);
        hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8t-1]=hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8t-1]+ex[-(c7t+7)+(c9t+3)-1][1+-(c7t+7)+c8t-1]-ex[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8t-1]-ey[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8t-1]);
        hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+(c8t+1)-1]=hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+3)-1][-(c7t+7)+(c8t+1)-1]+ex[-(c7t+7)+(c9t+3)-1][1+-(c7t+7)+(c8t+1)-1]-ex[-(c7t+7)+(c9t+3)-1][-(c7t+7)+(c8t+1)-1]-ey[-(c7t+7)+(c9t+3)-1][-(c7t+7)+(c8t+1)-1]);
      }
      register int cbv_3;
      cbv_3=32*c2+31;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_3; c8=c8+1) {
        ey[-c7t+c9t][-c7t+c8]=ey[-c7t+c9t][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8]-hz[-c7t+c9t-1][-c7t+c8]);
        ey[-c7t+(c9t+1)][-c7t+c8]=ey[-c7t+(c9t+1)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8]-hz[-c7t+(c9t+1)-1][-c7t+c8]);
        ey[-c7t+(c9t+2)][-c7t+c8]=ey[-c7t+(c9t+2)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8]-hz[-c7t+(c9t+2)-1][-c7t+c8]);
        ey[-c7t+(c9t+3)][-c7t+c8]=ey[-c7t+(c9t+3)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+3)][-c7t+c8]-hz[-c7t+(c9t+3)-1][-c7t+c8]);
        ey[-(c7t+1)+c9t][-(c7t+1)+c8]=ey[-(c7t+1)+c9t][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8]-hz[-(c7t+1)+c9t-1][-(c7t+1)+c8]);
        ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]=ey[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8]);
        ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]=ey[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8]);
        ey[-(c7t+1)+(c9t+3)][-(c7t+1)+c8]=ey[-(c7t+1)+(c9t+3)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+3)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8]);
        ey[-(c7t+2)+c9t][-(c7t+2)+c8]=ey[-(c7t+2)+c9t][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8]-hz[-(c7t+2)+c9t-1][-(c7t+2)+c8]);
        ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]=ey[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8]);
        ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]=ey[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8]);
        ey[-(c7t+2)+(c9t+3)][-(c7t+2)+c8]=ey[-(c7t+2)+(c9t+3)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+3)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8]);
        ey[-(c7t+3)+c9t][-(c7t+3)+c8]=ey[-(c7t+3)+c9t][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8]-hz[-(c7t+3)+c9t-1][-(c7t+3)+c8]);
        ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]=ey[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8]);
        ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]=ey[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8]);
        ey[-(c7t+3)+(c9t+3)][-(c7t+3)+c8]=ey[-(c7t+3)+(c9t+3)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+3)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8]);
        ey[-(c7t+4)+c9t][-(c7t+4)+c8]=ey[-(c7t+4)+c9t][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8]-hz[-(c7t+4)+c9t-1][-(c7t+4)+c8]);
        ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]=ey[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8]);
        ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]=ey[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8]);
        ey[-(c7t+4)+(c9t+3)][-(c7t+4)+c8]=ey[-(c7t+4)+(c9t+3)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+3)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8]);
        ey[-(c7t+5)+c9t][-(c7t+5)+c8]=ey[-(c7t+5)+c9t][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8]-hz[-(c7t+5)+c9t-1][-(c7t+5)+c8]);
        ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]=ey[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8]);
        ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]=ey[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8]);
        ey[-(c7t+5)+(c9t+3)][-(c7t+5)+c8]=ey[-(c7t+5)+(c9t+3)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+3)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8]);
        ey[-(c7t+6)+c9t][-(c7t+6)+c8]=ey[-(c7t+6)+c9t][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8]-hz[-(c7t+6)+c9t-1][-(c7t+6)+c8]);
        ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]=ey[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8]);
        ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]=ey[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8]);
        ey[-(c7t+6)+(c9t+3)][-(c7t+6)+c8]=ey[-(c7t+6)+(c9t+3)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+3)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8]);
        ey[-(c7t+7)+c9t][-(c7t+7)+c8]=ey[-(c7t+7)+c9t][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+c9t][-(c7t+7)+c8]-hz[-(c7t+7)+c9t-1][-(c7t+7)+c8]);
        ey[-(c7t+7)+(c9t+1)][-(c7t+7)+c8]=ey[-(c7t+7)+(c9t+1)][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+(c9t+1)][-(c7t+7)+c8]-hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8]);
        ey[-(c7t+7)+(c9t+2)][-(c7t+7)+c8]=ey[-(c7t+7)+(c9t+2)][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+(c9t+2)][-(c7t+7)+c8]-hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8]);
        ey[-(c7t+7)+(c9t+3)][-(c7t+7)+c8]=ey[-(c7t+7)+(c9t+3)][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+(c9t+3)][-(c7t+7)+c8]-hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8]);
        ex[-c7t+c9t][-c7t+c8]=ex[-c7t+c9t][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9t][-c7t+c8]-hz[-c7t+c9t][-c7t+c8-1]);
        ex[-c7t+(c9t+1)][-c7t+c8]=ex[-c7t+(c9t+1)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+1)][-c7t+c8]-hz[-c7t+(c9t+1)][-c7t+c8-1]);
        ex[-c7t+(c9t+2)][-c7t+c8]=ex[-c7t+(c9t+2)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+2)][-c7t+c8]-hz[-c7t+(c9t+2)][-c7t+c8-1]);
        ex[-c7t+(c9t+3)][-c7t+c8]=ex[-c7t+(c9t+3)][-c7t+c8]-((double)(1))/2*(hz[-c7t+(c9t+3)][-c7t+c8]-hz[-c7t+(c9t+3)][-c7t+c8-1]);
        ex[-(c7t+1)+c9t][-(c7t+1)+c8]=ex[-(c7t+1)+c9t][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9t][-(c7t+1)+c8]-hz[-(c7t+1)+c9t][-(c7t+1)+c8-1]);
        ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]=ex[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+1)][-(c7t+1)+c8-1]);
        ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]=ex[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+2)][-(c7t+1)+c8-1]);
        ex[-(c7t+1)+(c9t+3)][-(c7t+1)+c8]=ex[-(c7t+1)+(c9t+3)][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+(c9t+3)][-(c7t+1)+c8]-hz[-(c7t+1)+(c9t+3)][-(c7t+1)+c8-1]);
        ex[-(c7t+2)+c9t][-(c7t+2)+c8]=ex[-(c7t+2)+c9t][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9t][-(c7t+2)+c8]-hz[-(c7t+2)+c9t][-(c7t+2)+c8-1]);
        ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]=ex[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+1)][-(c7t+2)+c8-1]);
        ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]=ex[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+2)][-(c7t+2)+c8-1]);
        ex[-(c7t+2)+(c9t+3)][-(c7t+2)+c8]=ex[-(c7t+2)+(c9t+3)][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+(c9t+3)][-(c7t+2)+c8]-hz[-(c7t+2)+(c9t+3)][-(c7t+2)+c8-1]);
        ex[-(c7t+3)+c9t][-(c7t+3)+c8]=ex[-(c7t+3)+c9t][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9t][-(c7t+3)+c8]-hz[-(c7t+3)+c9t][-(c7t+3)+c8-1]);
        ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]=ex[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+1)][-(c7t+3)+c8-1]);
        ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]=ex[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+2)][-(c7t+3)+c8-1]);
        ex[-(c7t+3)+(c9t+3)][-(c7t+3)+c8]=ex[-(c7t+3)+(c9t+3)][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+(c9t+3)][-(c7t+3)+c8]-hz[-(c7t+3)+(c9t+3)][-(c7t+3)+c8-1]);
        ex[-(c7t+4)+c9t][-(c7t+4)+c8]=ex[-(c7t+4)+c9t][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9t][-(c7t+4)+c8]-hz[-(c7t+4)+c9t][-(c7t+4)+c8-1]);
        ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]=ex[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+1)][-(c7t+4)+c8-1]);
        ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]=ex[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+2)][-(c7t+4)+c8-1]);
        ex[-(c7t+4)+(c9t+3)][-(c7t+4)+c8]=ex[-(c7t+4)+(c9t+3)][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+(c9t+3)][-(c7t+4)+c8]-hz[-(c7t+4)+(c9t+3)][-(c7t+4)+c8-1]);
        ex[-(c7t+5)+c9t][-(c7t+5)+c8]=ex[-(c7t+5)+c9t][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9t][-(c7t+5)+c8]-hz[-(c7t+5)+c9t][-(c7t+5)+c8-1]);
        ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]=ex[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+1)][-(c7t+5)+c8-1]);
        ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]=ex[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+2)][-(c7t+5)+c8-1]);
        ex[-(c7t+5)+(c9t+3)][-(c7t+5)+c8]=ex[-(c7t+5)+(c9t+3)][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+(c9t+3)][-(c7t+5)+c8]-hz[-(c7t+5)+(c9t+3)][-(c7t+5)+c8-1]);
        ex[-(c7t+6)+c9t][-(c7t+6)+c8]=ex[-(c7t+6)+c9t][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9t][-(c7t+6)+c8]-hz[-(c7t+6)+c9t][-(c7t+6)+c8-1]);
        ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]=ex[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+1)][-(c7t+6)+c8-1]);
        ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]=ex[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+2)][-(c7t+6)+c8-1]);
        ex[-(c7t+6)+(c9t+3)][-(c7t+6)+c8]=ex[-(c7t+6)+(c9t+3)][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+(c9t+3)][-(c7t+6)+c8]-hz[-(c7t+6)+(c9t+3)][-(c7t+6)+c8-1]);
        ex[-(c7t+7)+c9t][-(c7t+7)+c8]=ex[-(c7t+7)+c9t][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+c9t][-(c7t+7)+c8]-hz[-(c7t+7)+c9t][-(c7t+7)+c8-1]);
        ex[-(c7t+7)+(c9t+1)][-(c7t+7)+c8]=ex[-(c7t+7)+(c9t+1)][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+(c9t+1)][-(c7t+7)+c8]-hz[-(c7t+7)+(c9t+1)][-(c7t+7)+c8-1]);
        ex[-(c7t+7)+(c9t+2)][-(c7t+7)+c8]=ex[-(c7t+7)+(c9t+2)][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+(c9t+2)][-(c7t+7)+c8]-hz[-(c7t+7)+(c9t+2)][-(c7t+7)+c8-1]);
        ex[-(c7t+7)+(c9t+3)][-(c7t+7)+c8]=ex[-(c7t+7)+(c9t+3)][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+(c9t+3)][-(c7t+7)+c8]-hz[-(c7t+7)+(c9t+3)][-(c7t+7)+c8-1]);
        hz[-c7t+c9t-1][-c7t+c8-1]=hz[-c7t+c9t-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+c9t-1][-c7t+c8-1]+ex[-c7t+c9t-1][1+-c7t+c8-1]-ex[-c7t+c9t-1][-c7t+c8-1]-ey[-c7t+c9t-1][-c7t+c8-1]);
        hz[-c7t+(c9t+1)-1][-c7t+c8-1]=hz[-c7t+(c9t+1)-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+(c9t+1)-1][-c7t+c8-1]+ex[-c7t+(c9t+1)-1][1+-c7t+c8-1]-ex[-c7t+(c9t+1)-1][-c7t+c8-1]-ey[-c7t+(c9t+1)-1][-c7t+c8-1]);
        hz[-c7t+(c9t+2)-1][-c7t+c8-1]=hz[-c7t+(c9t+2)-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+(c9t+2)-1][-c7t+c8-1]+ex[-c7t+(c9t+2)-1][1+-c7t+c8-1]-ex[-c7t+(c9t+2)-1][-c7t+c8-1]-ey[-c7t+(c9t+2)-1][-c7t+c8-1]);
        hz[-c7t+(c9t+3)-1][-c7t+c8-1]=hz[-c7t+(c9t+3)-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+(c9t+3)-1][-c7t+c8-1]+ex[-c7t+(c9t+3)-1][1+-c7t+c8-1]-ex[-c7t+(c9t+3)-1][-c7t+c8-1]-ey[-c7t+(c9t+3)-1][-c7t+c8-1]);
        hz[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9t-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+c9t-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+c9t-1][-(c7t+1)+c8-1]);
        hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+(c9t+1)-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+(c9t+1)-1][-(c7t+1)+c8-1]);
        hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+(c9t+2)-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+(c9t+2)-1][-(c7t+1)+c8-1]);
        hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+(c9t+3)-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+(c9t+3)-1][-(c7t+1)+c8-1]);
        hz[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9t-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+c9t-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+c9t-1][-(c7t+2)+c8-1]);
        hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+(c9t+1)-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+(c9t+1)-1][-(c7t+2)+c8-1]);
        hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+(c9t+2)-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+(c9t+2)-1][-(c7t+2)+c8-1]);
        hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+(c9t+3)-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+(c9t+3)-1][-(c7t+2)+c8-1]);
        hz[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9t-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+c9t-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+c9t-1][-(c7t+3)+c8-1]);
        hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+(c9t+1)-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+(c9t+1)-1][-(c7t+3)+c8-1]);
        hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+(c9t+2)-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+(c9t+2)-1][-(c7t+3)+c8-1]);
        hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+(c9t+3)-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+(c9t+3)-1][-(c7t+3)+c8-1]);
        hz[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9t-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+c9t-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+c9t-1][-(c7t+4)+c8-1]);
        hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+(c9t+1)-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+(c9t+1)-1][-(c7t+4)+c8-1]);
        hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+(c9t+2)-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+(c9t+2)-1][-(c7t+4)+c8-1]);
        hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+(c9t+3)-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+(c9t+3)-1][-(c7t+4)+c8-1]);
        hz[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9t-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+c9t-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+c9t-1][-(c7t+5)+c8-1]);
        hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+(c9t+1)-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+(c9t+1)-1][-(c7t+5)+c8-1]);
        hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+(c9t+2)-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+(c9t+2)-1][-(c7t+5)+c8-1]);
        hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+(c9t+3)-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+(c9t+3)-1][-(c7t+5)+c8-1]);
        hz[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9t-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+c9t-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+c9t-1][-(c7t+6)+c8-1]);
        hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+(c9t+1)-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+(c9t+1)-1][-(c7t+6)+c8-1]);
        hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+(c9t+2)-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+(c9t+2)-1][-(c7t+6)+c8-1]);
        hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+(c9t+3)-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+(c9t+3)-1][-(c7t+6)+c8-1]);
        hz[-(c7t+7)+c9t-1][-(c7t+7)+c8-1]=hz[-(c7t+7)+c9t-1][-(c7t+7)+c8-1]-((double)(7))/10*(ey[1+-(c7t+7)+c9t-1][-(c7t+7)+c8-1]+ex[-(c7t+7)+c9t-1][1+-(c7t+7)+c8-1]-ex[-(c7t+7)+c9t-1][-(c7t+7)+c8-1]-ey[-(c7t+7)+c9t-1][-(c7t+7)+c8-1]);
        hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8-1]=hz[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8-1]+ex[-(c7t+7)+(c9t+1)-1][1+-(c7t+7)+c8-1]-ex[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8-1]-ey[-(c7t+7)+(c9t+1)-1][-(c7t+7)+c8-1]);
        hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8-1]=hz[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8-1]+ex[-(c7t+7)+(c9t+2)-1][1+-(c7t+7)+c8-1]-ex[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8-1]-ey[-(c7t+7)+(c9t+2)-1][-(c7t+7)+c8-1]);
        hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8-1]=hz[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8-1]-((double)(7))/10*(ey[1+-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8-1]+ex[-(c7t+7)+(c9t+3)-1][1+-(c7t+7)+c8-1]-ex[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8-1]-ey[-(c7t+7)+(c9t+3)-1][-(c7t+7)+c8-1]);
      }
    }
    for (c9=c9t; c9<=32*c3+31; c9=c9+1) {
      register int cbv_4, cbv_5;
      cbv_4=32*c2;
      cbv_5=32*c2+30;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_4; c8t<=cbv_5; c8t=c8t+2) {
        ey[-c7t+c9][-c7t+c8t]=ey[-c7t+c9][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8t]-hz[-c7t+c9-1][-c7t+c8t]);
        ey[-c7t+c9][-c7t+(c8t+1)]=ey[-c7t+c9][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9][-c7t+(c8t+1)]-hz[-c7t+c9-1][-c7t+(c8t+1)]);
        ey[-(c7t+1)+c9][-(c7t+1)+c8t]=ey[-(c7t+1)+c9][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8t]-hz[-(c7t+1)+c9-1][-(c7t+1)+c8t]);
        ey[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]=ey[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)]);
        ey[-(c7t+2)+c9][-(c7t+2)+c8t]=ey[-(c7t+2)+c9][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8t]-hz[-(c7t+2)+c9-1][-(c7t+2)+c8t]);
        ey[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]=ey[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)]);
        ey[-(c7t+3)+c9][-(c7t+3)+c8t]=ey[-(c7t+3)+c9][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8t]-hz[-(c7t+3)+c9-1][-(c7t+3)+c8t]);
        ey[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]=ey[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)]);
        ey[-(c7t+4)+c9][-(c7t+4)+c8t]=ey[-(c7t+4)+c9][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8t]-hz[-(c7t+4)+c9-1][-(c7t+4)+c8t]);
        ey[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]=ey[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)]);
        ey[-(c7t+5)+c9][-(c7t+5)+c8t]=ey[-(c7t+5)+c9][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8t]-hz[-(c7t+5)+c9-1][-(c7t+5)+c8t]);
        ey[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]=ey[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)]);
        ey[-(c7t+6)+c9][-(c7t+6)+c8t]=ey[-(c7t+6)+c9][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8t]-hz[-(c7t+6)+c9-1][-(c7t+6)+c8t]);
        ey[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]=ey[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)]);
        ey[-(c7t+7)+c9][-(c7t+7)+c8t]=ey[-(c7t+7)+c9][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+c9][-(c7t+7)+c8t]-hz[-(c7t+7)+c9-1][-(c7t+7)+c8t]);
        ey[-(c7t+7)+c9][-(c7t+7)+(c8t+1)]=ey[-(c7t+7)+c9][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+c9][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+c9-1][-(c7t+7)+(c8t+1)]);
        ex[-c7t+c9][-c7t+c8t]=ex[-c7t+c9][-c7t+c8t]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8t]-hz[-c7t+c9][-c7t+c8t-1]);
        ex[-c7t+c9][-c7t+(c8t+1)]=ex[-c7t+c9][-c7t+(c8t+1)]-((double)(1))/2*(hz[-c7t+c9][-c7t+(c8t+1)]-hz[-c7t+c9][-c7t+(c8t+1)-1]);
        ex[-(c7t+1)+c9][-(c7t+1)+c8t]=ex[-(c7t+1)+c9][-(c7t+1)+c8t]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8t]-hz[-(c7t+1)+c9][-(c7t+1)+c8t-1]);
        ex[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]=ex[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+(c8t+1)]-hz[-(c7t+1)+c9][-(c7t+1)+(c8t+1)-1]);
        ex[-(c7t+2)+c9][-(c7t+2)+c8t]=ex[-(c7t+2)+c9][-(c7t+2)+c8t]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8t]-hz[-(c7t+2)+c9][-(c7t+2)+c8t-1]);
        ex[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]=ex[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+(c8t+1)]-hz[-(c7t+2)+c9][-(c7t+2)+(c8t+1)-1]);
        ex[-(c7t+3)+c9][-(c7t+3)+c8t]=ex[-(c7t+3)+c9][-(c7t+3)+c8t]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8t]-hz[-(c7t+3)+c9][-(c7t+3)+c8t-1]);
        ex[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]=ex[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+(c8t+1)]-hz[-(c7t+3)+c9][-(c7t+3)+(c8t+1)-1]);
        ex[-(c7t+4)+c9][-(c7t+4)+c8t]=ex[-(c7t+4)+c9][-(c7t+4)+c8t]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8t]-hz[-(c7t+4)+c9][-(c7t+4)+c8t-1]);
        ex[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]=ex[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+(c8t+1)]-hz[-(c7t+4)+c9][-(c7t+4)+(c8t+1)-1]);
        ex[-(c7t+5)+c9][-(c7t+5)+c8t]=ex[-(c7t+5)+c9][-(c7t+5)+c8t]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8t]-hz[-(c7t+5)+c9][-(c7t+5)+c8t-1]);
        ex[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]=ex[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+(c8t+1)]-hz[-(c7t+5)+c9][-(c7t+5)+(c8t+1)-1]);
        ex[-(c7t+6)+c9][-(c7t+6)+c8t]=ex[-(c7t+6)+c9][-(c7t+6)+c8t]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8t]-hz[-(c7t+6)+c9][-(c7t+6)+c8t-1]);
        ex[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]=ex[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+(c8t+1)]-hz[-(c7t+6)+c9][-(c7t+6)+(c8t+1)-1]);
        ex[-(c7t+7)+c9][-(c7t+7)+c8t]=ex[-(c7t+7)+c9][-(c7t+7)+c8t]-((double)(1))/2*(hz[-(c7t+7)+c9][-(c7t+7)+c8t]-hz[-(c7t+7)+c9][-(c7t+7)+c8t-1]);
        ex[-(c7t+7)+c9][-(c7t+7)+(c8t+1)]=ex[-(c7t+7)+c9][-(c7t+7)+(c8t+1)]-((double)(1))/2*(hz[-(c7t+7)+c9][-(c7t+7)+(c8t+1)]-hz[-(c7t+7)+c9][-(c7t+7)+(c8t+1)-1]);
        hz[-c7t+c9-1][-c7t+c8t-1]=hz[-c7t+c9-1][-c7t+c8t-1]-((double)(7))/10*(ey[1+-c7t+c9-1][-c7t+c8t-1]+ex[-c7t+c9-1][1+-c7t+c8t-1]-ex[-c7t+c9-1][-c7t+c8t-1]-ey[-c7t+c9-1][-c7t+c8t-1]);
        hz[-c7t+c9-1][-c7t+(c8t+1)-1]=hz[-c7t+c9-1][-c7t+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7t+c9-1][-c7t+(c8t+1)-1]+ex[-c7t+c9-1][1+-c7t+(c8t+1)-1]-ex[-c7t+c9-1][-c7t+(c8t+1)-1]-ey[-c7t+c9-1][-c7t+(c8t+1)-1]);
        hz[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]=hz[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9-1][-(c7t+1)+c8t-1]+ex[-(c7t+1)+c9-1][1+-(c7t+1)+c8t-1]-ex[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]-ey[-(c7t+1)+c9-1][-(c7t+1)+c8t-1]);
        hz[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]=hz[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]+ex[-(c7t+1)+c9-1][1+-(c7t+1)+(c8t+1)-1]-ex[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]-ey[-(c7t+1)+c9-1][-(c7t+1)+(c8t+1)-1]);
        hz[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]=hz[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9-1][-(c7t+2)+c8t-1]+ex[-(c7t+2)+c9-1][1+-(c7t+2)+c8t-1]-ex[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]-ey[-(c7t+2)+c9-1][-(c7t+2)+c8t-1]);
        hz[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]=hz[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]+ex[-(c7t+2)+c9-1][1+-(c7t+2)+(c8t+1)-1]-ex[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]-ey[-(c7t+2)+c9-1][-(c7t+2)+(c8t+1)-1]);
        hz[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]=hz[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9-1][-(c7t+3)+c8t-1]+ex[-(c7t+3)+c9-1][1+-(c7t+3)+c8t-1]-ex[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]-ey[-(c7t+3)+c9-1][-(c7t+3)+c8t-1]);
        hz[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]=hz[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]+ex[-(c7t+3)+c9-1][1+-(c7t+3)+(c8t+1)-1]-ex[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]-ey[-(c7t+3)+c9-1][-(c7t+3)+(c8t+1)-1]);
        hz[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]=hz[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9-1][-(c7t+4)+c8t-1]+ex[-(c7t+4)+c9-1][1+-(c7t+4)+c8t-1]-ex[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]-ey[-(c7t+4)+c9-1][-(c7t+4)+c8t-1]);
        hz[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]=hz[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]+ex[-(c7t+4)+c9-1][1+-(c7t+4)+(c8t+1)-1]-ex[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]-ey[-(c7t+4)+c9-1][-(c7t+4)+(c8t+1)-1]);
        hz[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]=hz[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9-1][-(c7t+5)+c8t-1]+ex[-(c7t+5)+c9-1][1+-(c7t+5)+c8t-1]-ex[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]-ey[-(c7t+5)+c9-1][-(c7t+5)+c8t-1]);
        hz[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]=hz[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]+ex[-(c7t+5)+c9-1][1+-(c7t+5)+(c8t+1)-1]-ex[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]-ey[-(c7t+5)+c9-1][-(c7t+5)+(c8t+1)-1]);
        hz[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]=hz[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9-1][-(c7t+6)+c8t-1]+ex[-(c7t+6)+c9-1][1+-(c7t+6)+c8t-1]-ex[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]-ey[-(c7t+6)+c9-1][-(c7t+6)+c8t-1]);
        hz[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]=hz[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]+ex[-(c7t+6)+c9-1][1+-(c7t+6)+(c8t+1)-1]-ex[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]-ey[-(c7t+6)+c9-1][-(c7t+6)+(c8t+1)-1]);
        hz[-(c7t+7)+c9-1][-(c7t+7)+c8t-1]=hz[-(c7t+7)+c9-1][-(c7t+7)+c8t-1]-((double)(7))/10*(ey[1+-(c7t+7)+c9-1][-(c7t+7)+c8t-1]+ex[-(c7t+7)+c9-1][1+-(c7t+7)+c8t-1]-ex[-(c7t+7)+c9-1][-(c7t+7)+c8t-1]-ey[-(c7t+7)+c9-1][-(c7t+7)+c8t-1]);
        hz[-(c7t+7)+c9-1][-(c7t+7)+(c8t+1)-1]=hz[-(c7t+7)+c9-1][-(c7t+7)+(c8t+1)-1]-((double)(7))/10*(ey[1+-(c7t+7)+c9-1][-(c7t+7)+(c8t+1)-1]+ex[-(c7t+7)+c9-1][1+-(c7t+7)+(c8t+1)-1]-ex[-(c7t+7)+c9-1][-(c7t+7)+(c8t+1)-1]-ey[-(c7t+7)+c9-1][-(c7t+7)+(c8t+1)-1]);
      }
      register int cbv_6;
      cbv_6=32*c2+31;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_6; c8=c8+1) {
        ey[-c7t+c9][-c7t+c8]=ey[-c7t+c9][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8]-hz[-c7t+c9-1][-c7t+c8]);
        ey[-(c7t+1)+c9][-(c7t+1)+c8]=ey[-(c7t+1)+c9][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8]-hz[-(c7t+1)+c9-1][-(c7t+1)+c8]);
        ey[-(c7t+2)+c9][-(c7t+2)+c8]=ey[-(c7t+2)+c9][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8]-hz[-(c7t+2)+c9-1][-(c7t+2)+c8]);
        ey[-(c7t+3)+c9][-(c7t+3)+c8]=ey[-(c7t+3)+c9][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8]-hz[-(c7t+3)+c9-1][-(c7t+3)+c8]);
        ey[-(c7t+4)+c9][-(c7t+4)+c8]=ey[-(c7t+4)+c9][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8]-hz[-(c7t+4)+c9-1][-(c7t+4)+c8]);
        ey[-(c7t+5)+c9][-(c7t+5)+c8]=ey[-(c7t+5)+c9][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8]-hz[-(c7t+5)+c9-1][-(c7t+5)+c8]);
        ey[-(c7t+6)+c9][-(c7t+6)+c8]=ey[-(c7t+6)+c9][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8]-hz[-(c7t+6)+c9-1][-(c7t+6)+c8]);
        ey[-(c7t+7)+c9][-(c7t+7)+c8]=ey[-(c7t+7)+c9][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+c9][-(c7t+7)+c8]-hz[-(c7t+7)+c9-1][-(c7t+7)+c8]);
        ex[-c7t+c9][-c7t+c8]=ex[-c7t+c9][-c7t+c8]-((double)(1))/2*(hz[-c7t+c9][-c7t+c8]-hz[-c7t+c9][-c7t+c8-1]);
        ex[-(c7t+1)+c9][-(c7t+1)+c8]=ex[-(c7t+1)+c9][-(c7t+1)+c8]-((double)(1))/2*(hz[-(c7t+1)+c9][-(c7t+1)+c8]-hz[-(c7t+1)+c9][-(c7t+1)+c8-1]);
        ex[-(c7t+2)+c9][-(c7t+2)+c8]=ex[-(c7t+2)+c9][-(c7t+2)+c8]-((double)(1))/2*(hz[-(c7t+2)+c9][-(c7t+2)+c8]-hz[-(c7t+2)+c9][-(c7t+2)+c8-1]);
        ex[-(c7t+3)+c9][-(c7t+3)+c8]=ex[-(c7t+3)+c9][-(c7t+3)+c8]-((double)(1))/2*(hz[-(c7t+3)+c9][-(c7t+3)+c8]-hz[-(c7t+3)+c9][-(c7t+3)+c8-1]);
        ex[-(c7t+4)+c9][-(c7t+4)+c8]=ex[-(c7t+4)+c9][-(c7t+4)+c8]-((double)(1))/2*(hz[-(c7t+4)+c9][-(c7t+4)+c8]-hz[-(c7t+4)+c9][-(c7t+4)+c8-1]);
        ex[-(c7t+5)+c9][-(c7t+5)+c8]=ex[-(c7t+5)+c9][-(c7t+5)+c8]-((double)(1))/2*(hz[-(c7t+5)+c9][-(c7t+5)+c8]-hz[-(c7t+5)+c9][-(c7t+5)+c8-1]);
        ex[-(c7t+6)+c9][-(c7t+6)+c8]=ex[-(c7t+6)+c9][-(c7t+6)+c8]-((double)(1))/2*(hz[-(c7t+6)+c9][-(c7t+6)+c8]-hz[-(c7t+6)+c9][-(c7t+6)+c8-1]);
        ex[-(c7t+7)+c9][-(c7t+7)+c8]=ex[-(c7t+7)+c9][-(c7t+7)+c8]-((double)(1))/2*(hz[-(c7t+7)+c9][-(c7t+7)+c8]-hz[-(c7t+7)+c9][-(c7t+7)+c8-1]);
        hz[-c7t+c9-1][-c7t+c8-1]=hz[-c7t+c9-1][-c7t+c8-1]-((double)(7))/10*(ey[1+-c7t+c9-1][-c7t+c8-1]+ex[-c7t+c9-1][1+-c7t+c8-1]-ex[-c7t+c9-1][-c7t+c8-1]-ey[-c7t+c9-1][-c7t+c8-1]);
        hz[-(c7t+1)+c9-1][-(c7t+1)+c8-1]=hz[-(c7t+1)+c9-1][-(c7t+1)+c8-1]-((double)(7))/10*(ey[1+-(c7t+1)+c9-1][-(c7t+1)+c8-1]+ex[-(c7t+1)+c9-1][1+-(c7t+1)+c8-1]-ex[-(c7t+1)+c9-1][-(c7t+1)+c8-1]-ey[-(c7t+1)+c9-1][-(c7t+1)+c8-1]);
        hz[-(c7t+2)+c9-1][-(c7t+2)+c8-1]=hz[-(c7t+2)+c9-1][-(c7t+2)+c8-1]-((double)(7))/10*(ey[1+-(c7t+2)+c9-1][-(c7t+2)+c8-1]+ex[-(c7t+2)+c9-1][1+-(c7t+2)+c8-1]-ex[-(c7t+2)+c9-1][-(c7t+2)+c8-1]-ey[-(c7t+2)+c9-1][-(c7t+2)+c8-1]);
        hz[-(c7t+3)+c9-1][-(c7t+3)+c8-1]=hz[-(c7t+3)+c9-1][-(c7t+3)+c8-1]-((double)(7))/10*(ey[1+-(c7t+3)+c9-1][-(c7t+3)+c8-1]+ex[-(c7t+3)+c9-1][1+-(c7t+3)+c8-1]-ex[-(c7t+3)+c9-1][-(c7t+3)+c8-1]-ey[-(c7t+3)+c9-1][-(c7t+3)+c8-1]);
        hz[-(c7t+4)+c9-1][-(c7t+4)+c8-1]=hz[-(c7t+4)+c9-1][-(c7t+4)+c8-1]-((double)(7))/10*(ey[1+-(c7t+4)+c9-1][-(c7t+4)+c8-1]+ex[-(c7t+4)+c9-1][1+-(c7t+4)+c8-1]-ex[-(c7t+4)+c9-1][-(c7t+4)+c8-1]-ey[-(c7t+4)+c9-1][-(c7t+4)+c8-1]);
        hz[-(c7t+5)+c9-1][-(c7t+5)+c8-1]=hz[-(c7t+5)+c9-1][-(c7t+5)+c8-1]-((double)(7))/10*(ey[1+-(c7t+5)+c9-1][-(c7t+5)+c8-1]+ex[-(c7t+5)+c9-1][1+-(c7t+5)+c8-1]-ex[-(c7t+5)+c9-1][-(c7t+5)+c8-1]-ey[-(c7t+5)+c9-1][-(c7t+5)+c8-1]);
        hz[-(c7t+6)+c9-1][-(c7t+6)+c8-1]=hz[-(c7t+6)+c9-1][-(c7t+6)+c8-1]-((double)(7))/10*(ey[1+-(c7t+6)+c9-1][-(c7t+6)+c8-1]+ex[-(c7t+6)+c9-1][1+-(c7t+6)+c8-1]-ex[-(c7t+6)+c9-1][-(c7t+6)+c8-1]-ey[-(c7t+6)+c9-1][-(c7t+6)+c8-1]);
        hz[-(c7t+7)+c9-1][-(c7t+7)+c8-1]=hz[-(c7t+7)+c9-1][-(c7t+7)+c8-1]-((double)(7))/10*(ey[1+-(c7t+7)+c9-1][-(c7t+7)+c8-1]+ex[-(c7t+7)+c9-1][1+-(c7t+7)+c8-1]-ex[-(c7t+7)+c9-1][-(c7t+7)+c8-1]-ey[-(c7t+7)+c9-1][-(c7t+7)+c8-1]);
      }
    }
  }
  for (c7=c7t; c7<=min(min(min(tmax-1,32*c3-1),16*c1-16*c2+15),32*c2-1); c7=c7+1) {
    for (c9t=32*c3; c9t<=32*c3+28; c9t=c9t+4) {
      register int cbv_7, cbv_8;
      cbv_7=32*c2;
      cbv_8=32*c2+30;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_7; c8t<=cbv_8; c8t=c8t+2) {
        ey[-c7+c9t][-c7+c8t]=ey[-c7+c9t][-c7+c8t]-((double)(1))/2*(hz[-c7+c9t][-c7+c8t]-hz[-c7+c9t-1][-c7+c8t]);
        ey[-c7+c9t][-c7+(c8t+1)]=ey[-c7+c9t][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9t][-c7+(c8t+1)]-hz[-c7+c9t-1][-c7+(c8t+1)]);
        ey[-c7+(c9t+1)][-c7+c8t]=ey[-c7+(c9t+1)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8t]-hz[-c7+(c9t+1)-1][-c7+c8t]);
        ey[-c7+(c9t+1)][-c7+(c8t+1)]=ey[-c7+(c9t+1)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+(c8t+1)]-hz[-c7+(c9t+1)-1][-c7+(c8t+1)]);
        ey[-c7+(c9t+2)][-c7+c8t]=ey[-c7+(c9t+2)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8t]-hz[-c7+(c9t+2)-1][-c7+c8t]);
        ey[-c7+(c9t+2)][-c7+(c8t+1)]=ey[-c7+(c9t+2)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+(c8t+1)]-hz[-c7+(c9t+2)-1][-c7+(c8t+1)]);
        ey[-c7+(c9t+3)][-c7+c8t]=ey[-c7+(c9t+3)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+3)][-c7+c8t]-hz[-c7+(c9t+3)-1][-c7+c8t]);
        ey[-c7+(c9t+3)][-c7+(c8t+1)]=ey[-c7+(c9t+3)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+3)][-c7+(c8t+1)]-hz[-c7+(c9t+3)-1][-c7+(c8t+1)]);
        ex[-c7+c9t][-c7+c8t]=ex[-c7+c9t][-c7+c8t]-((double)(1))/2*(hz[-c7+c9t][-c7+c8t]-hz[-c7+c9t][-c7+c8t-1]);
        ex[-c7+c9t][-c7+(c8t+1)]=ex[-c7+c9t][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9t][-c7+(c8t+1)]-hz[-c7+c9t][-c7+(c8t+1)-1]);
        ex[-c7+(c9t+1)][-c7+c8t]=ex[-c7+(c9t+1)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8t]-hz[-c7+(c9t+1)][-c7+c8t-1]);
        ex[-c7+(c9t+1)][-c7+(c8t+1)]=ex[-c7+(c9t+1)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+(c8t+1)]-hz[-c7+(c9t+1)][-c7+(c8t+1)-1]);
        ex[-c7+(c9t+2)][-c7+c8t]=ex[-c7+(c9t+2)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8t]-hz[-c7+(c9t+2)][-c7+c8t-1]);
        ex[-c7+(c9t+2)][-c7+(c8t+1)]=ex[-c7+(c9t+2)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+(c8t+1)]-hz[-c7+(c9t+2)][-c7+(c8t+1)-1]);
        ex[-c7+(c9t+3)][-c7+c8t]=ex[-c7+(c9t+3)][-c7+c8t]-((double)(1))/2*(hz[-c7+(c9t+3)][-c7+c8t]-hz[-c7+(c9t+3)][-c7+c8t-1]);
        ex[-c7+(c9t+3)][-c7+(c8t+1)]=ex[-c7+(c9t+3)][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+(c9t+3)][-c7+(c8t+1)]-hz[-c7+(c9t+3)][-c7+(c8t+1)-1]);
        hz[-c7+c9t-1][-c7+c8t-1]=hz[-c7+c9t-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+c9t-1][-c7+c8t-1]+ex[-c7+c9t-1][1+-c7+c8t-1]-ex[-c7+c9t-1][-c7+c8t-1]-ey[-c7+c9t-1][-c7+c8t-1]);
        hz[-c7+c9t-1][-c7+(c8t+1)-1]=hz[-c7+c9t-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+c9t-1][-c7+(c8t+1)-1]+ex[-c7+c9t-1][1+-c7+(c8t+1)-1]-ex[-c7+c9t-1][-c7+(c8t+1)-1]-ey[-c7+c9t-1][-c7+(c8t+1)-1]);
        hz[-c7+(c9t+1)-1][-c7+c8t-1]=hz[-c7+(c9t+1)-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+(c9t+1)-1][-c7+c8t-1]+ex[-c7+(c9t+1)-1][1+-c7+c8t-1]-ex[-c7+(c9t+1)-1][-c7+c8t-1]-ey[-c7+(c9t+1)-1][-c7+c8t-1]);
        hz[-c7+(c9t+1)-1][-c7+(c8t+1)-1]=hz[-c7+(c9t+1)-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+(c9t+1)-1][-c7+(c8t+1)-1]+ex[-c7+(c9t+1)-1][1+-c7+(c8t+1)-1]-ex[-c7+(c9t+1)-1][-c7+(c8t+1)-1]-ey[-c7+(c9t+1)-1][-c7+(c8t+1)-1]);
        hz[-c7+(c9t+2)-1][-c7+c8t-1]=hz[-c7+(c9t+2)-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+(c9t+2)-1][-c7+c8t-1]+ex[-c7+(c9t+2)-1][1+-c7+c8t-1]-ex[-c7+(c9t+2)-1][-c7+c8t-1]-ey[-c7+(c9t+2)-1][-c7+c8t-1]);
        hz[-c7+(c9t+2)-1][-c7+(c8t+1)-1]=hz[-c7+(c9t+2)-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+(c9t+2)-1][-c7+(c8t+1)-1]+ex[-c7+(c9t+2)-1][1+-c7+(c8t+1)-1]-ex[-c7+(c9t+2)-1][-c7+(c8t+1)-1]-ey[-c7+(c9t+2)-1][-c7+(c8t+1)-1]);
        hz[-c7+(c9t+3)-1][-c7+c8t-1]=hz[-c7+(c9t+3)-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+(c9t+3)-1][-c7+c8t-1]+ex[-c7+(c9t+3)-1][1+-c7+c8t-1]-ex[-c7+(c9t+3)-1][-c7+c8t-1]-ey[-c7+(c9t+3)-1][-c7+c8t-1]);
        hz[-c7+(c9t+3)-1][-c7+(c8t+1)-1]=hz[-c7+(c9t+3)-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+(c9t+3)-1][-c7+(c8t+1)-1]+ex[-c7+(c9t+3)-1][1+-c7+(c8t+1)-1]-ex[-c7+(c9t+3)-1][-c7+(c8t+1)-1]-ey[-c7+(c9t+3)-1][-c7+(c8t+1)-1]);
      }
      register int cbv_9;
      cbv_9=32*c2+31;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_9; c8=c8+1) {
        ey[-c7+c9t][-c7+c8]=ey[-c7+c9t][-c7+c8]-((double)(1))/2*(hz[-c7+c9t][-c7+c8]-hz[-c7+c9t-1][-c7+c8]);
        ey[-c7+(c9t+1)][-c7+c8]=ey[-c7+(c9t+1)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8]-hz[-c7+(c9t+1)-1][-c7+c8]);
        ey[-c7+(c9t+2)][-c7+c8]=ey[-c7+(c9t+2)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8]-hz[-c7+(c9t+2)-1][-c7+c8]);
        ey[-c7+(c9t+3)][-c7+c8]=ey[-c7+(c9t+3)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+3)][-c7+c8]-hz[-c7+(c9t+3)-1][-c7+c8]);
        ex[-c7+c9t][-c7+c8]=ex[-c7+c9t][-c7+c8]-((double)(1))/2*(hz[-c7+c9t][-c7+c8]-hz[-c7+c9t][-c7+c8-1]);
        ex[-c7+(c9t+1)][-c7+c8]=ex[-c7+(c9t+1)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+1)][-c7+c8]-hz[-c7+(c9t+1)][-c7+c8-1]);
        ex[-c7+(c9t+2)][-c7+c8]=ex[-c7+(c9t+2)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+2)][-c7+c8]-hz[-c7+(c9t+2)][-c7+c8-1]);
        ex[-c7+(c9t+3)][-c7+c8]=ex[-c7+(c9t+3)][-c7+c8]-((double)(1))/2*(hz[-c7+(c9t+3)][-c7+c8]-hz[-c7+(c9t+3)][-c7+c8-1]);
        hz[-c7+c9t-1][-c7+c8-1]=hz[-c7+c9t-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9t-1][-c7+c8-1]+ex[-c7+c9t-1][1+-c7+c8-1]-ex[-c7+c9t-1][-c7+c8-1]-ey[-c7+c9t-1][-c7+c8-1]);
        hz[-c7+(c9t+1)-1][-c7+c8-1]=hz[-c7+(c9t+1)-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+(c9t+1)-1][-c7+c8-1]+ex[-c7+(c9t+1)-1][1+-c7+c8-1]-ex[-c7+(c9t+1)-1][-c7+c8-1]-ey[-c7+(c9t+1)-1][-c7+c8-1]);
        hz[-c7+(c9t+2)-1][-c7+c8-1]=hz[-c7+(c9t+2)-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+(c9t+2)-1][-c7+c8-1]+ex[-c7+(c9t+2)-1][1+-c7+c8-1]-ex[-c7+(c9t+2)-1][-c7+c8-1]-ey[-c7+(c9t+2)-1][-c7+c8-1]);
        hz[-c7+(c9t+3)-1][-c7+c8-1]=hz[-c7+(c9t+3)-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+(c9t+3)-1][-c7+c8-1]+ex[-c7+(c9t+3)-1][1+-c7+c8-1]-ex[-c7+(c9t+3)-1][-c7+c8-1]-ey[-c7+(c9t+3)-1][-c7+c8-1]);
      }
    }
    for (c9=c9t; c9<=32*c3+31; c9=c9+1) {
      register int cbv_10, cbv_11;
      cbv_10=32*c2;
      cbv_11=32*c2+30;
#pragma ivdep
#pragma vector always
      for (c8t=cbv_10; c8t<=cbv_11; c8t=c8t+2) {
        ey[-c7+c9][-c7+c8t]=ey[-c7+c9][-c7+c8t]-((double)(1))/2*(hz[-c7+c9][-c7+c8t]-hz[-c7+c9-1][-c7+c8t]);
        ey[-c7+c9][-c7+(c8t+1)]=ey[-c7+c9][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9][-c7+(c8t+1)]-hz[-c7+c9-1][-c7+(c8t+1)]);
        ex[-c7+c9][-c7+c8t]=ex[-c7+c9][-c7+c8t]-((double)(1))/2*(hz[-c7+c9][-c7+c8t]-hz[-c7+c9][-c7+c8t-1]);
        ex[-c7+c9][-c7+(c8t+1)]=ex[-c7+c9][-c7+(c8t+1)]-((double)(1))/2*(hz[-c7+c9][-c7+(c8t+1)]-hz[-c7+c9][-c7+(c8t+1)-1]);
        hz[-c7+c9-1][-c7+c8t-1]=hz[-c7+c9-1][-c7+c8t-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8t-1]+ex[-c7+c9-1][1+-c7+c8t-1]-ex[-c7+c9-1][-c7+c8t-1]-ey[-c7+c9-1][-c7+c8t-1]);
        hz[-c7+c9-1][-c7+(c8t+1)-1]=hz[-c7+c9-1][-c7+(c8t+1)-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+(c8t+1)-1]+ex[-c7+c9-1][1+-c7+(c8t+1)-1]-ex[-c7+c9-1][-c7+(c8t+1)-1]-ey[-c7+c9-1][-c7+(c8t+1)-1]);
      }
      register int cbv_12;
      cbv_12=32*c2+31;
#pragma ivdep
#pragma vector always
      for (c8=c8t; c8<=cbv_12; c8=c8+1) {
        ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
        ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
        hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
      }
    }
  }
}
/*@ end @*/

      for (c7=max(max(max(max(32*c2,32*c3-nx+32),0),16*c1-16*c2),32*c2-ny+32);c7<=min(min(min(tmax-1,32*c3-1),16*c1-16*c2+15),32*c2+30);c7++) {
        for (c9=32*c3;c9<=32*c3+31;c9++) {
          ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
        }
        for (c8=c7+1;c8<=32*c2+31;c8++) {
          for (c9=32*c3;c9<=32*c3+31;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
      }
      if (ny == 1) {
        for (c7=max(max(max(0,32*c3-nx+1),16*c1-16*c2),32*c2);c7<=min(min(min(32*c3-1,tmax-1),16*c1-16*c2+15),32*c2+30);c7++) {
          for (c9=32*c3;c9<=min(c7+nx-1,32*c3+31);c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c9=32*c3;c9<=min(c7+nx,32*c3+31);c9++) {
            hz[-c7+c9-1][0]=hz[-c7+c9-1][0]-((double)(7))/10*(ey[1+-c7+c9-1][0]+ex[-c7+c9-1][1+0]-ex[-c7+c9-1][0]-ey[-c7+c9-1][0]);
          }
        }
      }
      if ((nx == 1) && (ny >= 2)) {
        for (c7=max(max(max(16*c3+16,32*c2-ny+1),32*c3),16*c1-16*c2);c7<=min(min(min(tmax-1,32*c2-ny+31),32*c3+30),16*c1-16*c2+15);c7++) {
          for (c8=max(c7+1,32*c2);c8<=c7+ny-1;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
          hz[0][ny-1]=hz[0][ny-1]-((double)(7))/10*(ey[1+0][ny-1]+ex[0][1+ny-1]-ex[0][ny-1]-ey[0][ny-1]);
        }
      }
      if (nx == 1) {
        for (c7=max(max(max(16*c3+16,32*c3),16*c1-16*c2),32*c2-ny+32);c7<=min(min(min(tmax-1,16*c1-16*c2+15),32*c2+30),32*c3+30);c7++) {
          for (c8=max(c7+1,32*c2);c8<=32*c2+31;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            hz[0][-c7+c8-1]=hz[0][-c7+c8-1]-((double)(7))/10*(ey[1+0][-c7+c8-1]+ex[0][1+-c7+c8-1]-ex[0][-c7+c8-1]-ey[0][-c7+c8-1]);
          }
        }
      }
      if ((nx == 1) && (ny == 1)) {
        for (c7=max(max(max(16*c3+16,16*c1-16*c2),32*c3),32*c2-1);c7<=min(min(min(tmax-1,32*c2+30),16*c1-16*c2+15),32*c3+30);c7++) {
          hz[0][0]=hz[0][0]-((double)(7))/10*(ey[1+0][0]+ex[0][1+0]-ex[0][0]-ey[0][0]);
        }
      }
      if ((c1 == c2+c3) && (nx >= 2)) {
        for (c7=max(max(32*c3,0),32*c2-ny+1);c7<=min(min(min(min(32*c2-1,32*c3-nx+31),tmax-1),16*c3+15),32*c2-ny+31);c7++) {
          for (c8=32*c2;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=c7+1;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((c1 == c2+c3) && (nx >= 2) && (ny >= 2)) {
        for (c7=max(max(32*c2,32*c3),0);c7<=min(min(min(32*c3-nx+31,tmax-1),16*c3+15),32*c2-ny+31);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=c7+1;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((c1 == c2+c3) && (nx >= 2)) {
        for (c7=max(max(32*c3,0),32*c2-ny+32);c7<=min(min(min(tmax-1,16*c3+15),32*c2-1),32*c3-nx+31);c7++) {
          for (c8=32*c2;c8<=32*c2+31;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
        }
      }
      if (c1 == c2+c3) {
        for (c7=max(max(max(32*c3-nx+32,32*c3),0),32*c2-ny+1);c7<=min(min(min(32*c2-1,tmax-1),16*c3+15),32*c2-ny+31);c7++) {
          for (c8=32*c2;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=32*c3+31;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((c1 == c2+c3) && (nx >= 2)) {
        for (c7=max(max(max(32*c2,32*c3),0),32*c2-ny+32);c7<=min(min(min(tmax-1,16*c3+15),32*c2+30),32*c3-nx+31);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=32*c2+31;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
        }
      }
      if ((c1 == c2+c3) && (ny >= 2)) {
        for (c7=max(max(max(32*c2,32*c3-nx+32),32*c3),0);c7<=min(min(tmax-1,16*c3+15),32*c2-ny+31);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=32*c3+31;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if (c1 == c2+c3) {
        for (c7=max(max(max(32*c3,0),32*c3-nx+32),32*c2-ny+32);c7<=min(min(tmax-1,16*c3+15),32*c2-1);c7++) {
          for (c8=32*c2;c8<=32*c2+31;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=32*c3+31;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
        }
      }
      if (c1 == c2+c3) {
        for (c7=max(max(max(max(32*c2,32*c3-nx+32),32*c3),0),32*c2-ny+32);c7<=min(min(tmax-1,16*c3+15),32*c2+30);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=32*c2+31;c8++) {
            ey[0][-c7+c8]=c7;
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=32*c3+31;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
        }
      }
      if ((c1 == c2+c3) && (nx >= 2) && (ny == 1)) {
        for (c7=max(max(32*c3,0),32*c2);c7<=min(min(tmax-1,16*c3+15),32*c2+30);c7++) {
          ey[0][0]=c7;
          for (c9=c7+1;c9<=min(c7+nx-1,32*c3+31);c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c9=c7+1;c9<=min(c7+nx,32*c3+31);c9++) {
            hz[-c7+c9-1][0]=hz[-c7+c9-1][0]-((double)(7))/10*(ey[1+-c7+c9-1][0]+ex[-c7+c9-1][1+0]-ex[-c7+c9-1][0]-ey[-c7+c9-1][0]);
          }
        }
      }
      if ((-c1 == -c2-c3) && (c1 <= min(floord(3*c3-1,2),floord(32*c3+tmax-32,32))) && (nx == 1) && (ny == 1)) {
        ey[0][0]=32*c1-32*c3+31;
      }
      if ((-c1 == -c2-c3) && (c1 <= min(floord(3*c3-1,2),floord(32*c3+tmax-32,32))) && (nx == 1) && (ny >= 2)) {
        ey[0][0]=32*c1-32*c3+31;
      }
      if ((c1 >= 3*c2+1) && (c2 <= min(floord(tmax-32,32),c3-1)) && (ny == 1)) {
        for (c9=32*c3;c9<=min(32*c2+nx+30,32*c3+31);c9++) {
          ey[-32*c2+c9-31][0]=ey[-32*c2+c9-31][0]-((double)(1))/2*(hz[-32*c2+c9-31][0]-hz[-32*c2+c9-31 -1][0]);
        }
      }
      if ((c1 >= 3*c2+1) && (c2 <= min(floord(tmax-32,32),c3-1)) && (ny >= 2)) {
        for (c9=32*c3;c9<=min(32*c2+nx+30,32*c3+31);c9++) {
          ey[-32*c2+c9-31][0]=ey[-32*c2+c9-31][0]-((double)(1))/2*(hz[-32*c2+c9-31][0]-hz[-32*c2+c9-31 -1][0]);
        }
      }
      if (nx >= 2) {
        for (c7=max(max(max(16*c3+16,32*c3),16*c1-16*c2),32*c2-ny+1);c7<=min(min(min(min(32*c2-1,32*c3-nx+31),tmax-1),32*c2-ny+31),16*c1-16*c2+15);c7++) {
          for (c8=32*c2;c8<=c7+ny-1;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=c7+1;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if ((nx >= 2) && (ny >= 2)) {
        for (c7=max(max(max(32*c2,16*c3+16),32*c3),16*c1-16*c2);c7<=min(min(min(32*c3-nx+31,tmax-1),32*c2-ny+31),16*c1-16*c2+15);c7++) {
          for (c9=c7+1;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
          for (c9=c7+1;c9<=c7+nx;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      if (nx >= 2) {
        for (c7=max(max(max(16*c3+16,16*c1-16*c2),32*c3),32*c2-ny+32);c7<=min(min(min(tmax-1,16*c1-16*c2+15),32*c2-1),32*c3-nx+31);c7++) {
          for (c8=32*c2;c8<=32*c2+31;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
        }
      }
      for (c7=max(max(max(max(32*c3-nx+32,16*c3+16),32*c3),16*c1-16*c2),32*c2-ny+1);c7<=min(min(min(min(32*c2-1,tmax-1),32*c2-ny+31),16*c1-16*c2+15),32*c3+30);c7++) {
        for (c8=32*c2;c8<=c7+ny-1;c8++) {
          ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
        for (c9=c7+1;c9<=32*c3+31;c9++) {
          hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
        }
      }
      if (nx >= 2) {
        for (c7=max(max(max(max(32*c2,16*c3+16),16*c1-16*c2),32*c3),32*c2-ny+32);c7<=min(min(min(tmax-1,32*c2+30),16*c1-16*c2+15),32*c3-nx+31);c7++) {
          for (c9=c7+1;c9<=c7+nx-1;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=32*c2+31;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=c7+nx-1;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
            hz[nx-1][-c7+c8-1]=hz[nx-1][-c7+c8-1]-((double)(7))/10*(ey[1+nx-1][-c7+c8-1]+ex[nx-1][1+-c7+c8-1]-ex[nx-1][-c7+c8-1]-ey[nx-1][-c7+c8-1]);
          }
        }
      }
      if (ny >= 2) {
        for (c7=max(max(max(max(32*c2,32*c3-nx+32),16*c3+16),32*c3),16*c1-16*c2);c7<=min(min(min(tmax-1,32*c2-ny+31),16*c1-16*c2+15),32*c3+30);c7++) {
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c8=c7+1;c8<=c7+ny-1;c8++) {
            ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
            for (c9=c7+1;c9<=32*c3+31;c9++) {
              ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
              ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
              hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
            }
          }
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            hz[-c7+c9-1][ny-1]=hz[-c7+c9-1][ny-1]-((double)(7))/10*(ey[1+-c7+c9-1][ny-1]+ex[-c7+c9-1][1+ny-1]-ex[-c7+c9-1][ny-1]-ey[-c7+c9-1][ny-1]);
          }
        }
      }
      for (c7=max(max(max(max(16*c3+16,16*c1-16*c2),32*c3),32*c3-nx+32),32*c2-ny+32);c7<=min(min(min(tmax-1,32*c3+30),16*c1-16*c2+15),32*c2-1);c7++) {
        for (c8=32*c2;c8<=32*c2+31;c8++) {
          ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
      }
      for (c7=max(max(max(max(max(32*c2,32*c3-nx+32),16*c3+16),16*c1-16*c2),32*c3),32*c2-ny+32);c7<=min(min(min(tmax-1,32*c3+30),16*c1-16*c2+15),32*c2+30);c7++) {
        for (c9=c7+1;c9<=32*c3+31;c9++) {
          ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
        }
        for (c8=c7+1;c8<=32*c2+31;c8++) {
          ex[0][-c7+c8]=ex[0][-c7+c8]-((double)(1))/2*(hz[0][-c7+c8]-hz[0][-c7+c8-1]);
          for (c9=c7+1;c9<=32*c3+31;c9++) {
            ey[-c7+c9][-c7+c8]=ey[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9-1][-c7+c8]);
            ex[-c7+c9][-c7+c8]=ex[-c7+c9][-c7+c8]-((double)(1))/2*(hz[-c7+c9][-c7+c8]-hz[-c7+c9][-c7+c8-1]);
            hz[-c7+c9-1][-c7+c8-1]=hz[-c7+c9-1][-c7+c8-1]-((double)(7))/10*(ey[1+-c7+c9-1][-c7+c8-1]+ex[-c7+c9-1][1+-c7+c8-1]-ex[-c7+c9-1][-c7+c8-1]-ey[-c7+c9-1][-c7+c8-1]);
          }
        }
      }
      if ((nx >= 2) && (ny == 1)) {
        for (c7=max(max(max(16*c3+16,16*c1-16*c2),32*c3),32*c2);c7<=min(min(min(tmax-1,32*c2+30),32*c3+30),16*c1-16*c2+15);c7++) {
          for (c9=c7+1;c9<=min(c7+nx-1,32*c3+31);c9++) {
            ey[-c7+c9][0]=ey[-c7+c9][0]-((double)(1))/2*(hz[-c7+c9][0]-hz[-c7+c9-1][0]);
          }
          for (c9=c7+1;c9<=min(32*c3+31,c7+nx);c9++) {
            hz[-c7+c9-1][0]=hz[-c7+c9-1][0]-((double)(7))/10*(ey[1+-c7+c9-1][0]+ex[-c7+c9-1][1+0]-ex[-c7+c9-1][0]-ey[-c7+c9-1][0]);
          }
        }
      }
      if ((c1 >= c2+2*c3+1) && (c2 >= ceild(32*c3+1,32)) && (c3 >= 0) && (c3 <= floord(tmax-32,32)) && (nx == 1) && (ny >= 2)) {
        for (c8=max(32*c3+32,32*c2);c8<=min(32*c3+ny+30,32*c2+31);c8++) {
          ex[0][-32*c3+c8-31]=ex[0][-32*c3+c8-31]-((double)(1))/2*(hz[0][-32*c3+c8-31]-hz[0][-32*c3+c8-31 -1]);
        }
      }
      if ((-c1 == -c2-c3) && (c1 <= min(floord(3*c3-1,2),floord(32*c3+tmax-32,32))) && (c1 >= ceild(64*c3-31,32)) && (nx >= 2) && (ny == 1)) {
        ey[0][0]=32*c1-32*c3+31;
        for (c9=32*c1-32*c3+32;c9<=min(32*c1-32*c3+nx+30,32*c3+31);c9++) {
          ey[-32*c1+32*c3+c9-31][0]=ey[-32*c1+32*c3+c9-31][0]-((double)(1))/2*(hz[-32*c1+32*c3+c9-31][0]-hz[-32*c1+32*c3+c9-31 -1][0]);
        }
      }
      if ((-c1 == -c2-c3) && (c1 <= min(floord(3*c3-1,2),floord(32*c3+tmax-32,32))) && (c1 >= ceild(64*c3-31,32)) && (nx >= 2) && (ny >= 2)) {
        ey[0][0]=32*c1-32*c3+31;
        for (c9=32*c1-32*c3+32;c9<=min(32*c1-32*c3+nx+30,32*c3+31);c9++) {
          ey[-32*c1+32*c3+c9-31][0]=ey[-32*c1+32*c3+c9-31][0]-((double)(1))/2*(hz[-32*c1+32*c3+c9-31][0]-hz[-32*c1+32*c3+c9-31 -1][0]);
        }
      }
      if ((c1 >= 3*c2+1) && (c2 >= max(ceild(16*c3-15,32),ceild(32*c3-31,32))) && (c2 <= min(floord(tmax-32,32),floord(32*c3-1,32))) && (nx >= 2) && (ny == 1)) {
        for (c9=32*c2+32;c9<=min(32*c2+nx+30,32*c3+31);c9++) {
          ey[-32*c2+c9-31][0]=ey[-32*c2+c9-31][0]-((double)(1))/2*(hz[-32*c2+c9-31][0]-hz[-32*c2+c9-31 -1][0]);
        }
      }
      if ((c1 >= 3*c2+1) && (c2 <= min(floord(tmax-32,32),floord(32*c3-1,32))) && (c2 >= max(ceild(16*c3-15,32),ceild(32*c3-31,32))) && (nx >= 2) && (ny >= 2)) {
        for (c9=32*c2+32;c9<=min(32*c2+nx+30,32*c3+31);c9++) {
          ey[-32*c2+c9-31][0]=ey[-32*c2+c9-31][0]-((double)(1))/2*(hz[-32*c2+c9-31][0]-hz[-32*c2+c9-31 -1][0]);
        }
      }
      if ((c1 >= c2+2*c3+1) && (c2 >= ceild(32*c3+1,32)) && (c3 >= 0) && (c3 <= floord(tmax-32,32)) && (nx >= 2) && (ny >= 2)) {
        for (c8=max(32*c3+32,32*c2);c8<=min(32*c3+ny+30,32*c2+31);c8++) {
          ex[0][-32*c3+c8-31]=ex[0][-32*c3+c8-31]-((double)(1))/2*(hz[0][-32*c3+c8-31]-hz[0][-32*c3+c8-31 -1]);
        }
      }
    }
  }
}
/* polysyn end */



/*@ end @*/
/*@ end @*/



    annot_t_end = rtclock();
    annot_t_total += annot_t_end - annot_t_start;
  }
  
  annot_t_total = annot_t_total / REPS;

#ifndef TEST
  printf("%f\n", annot_t_total);
#else
  {
    int i,j;
    for (i=0; i<nx; i++) {
      for (j=0; j<ny; j++)  {
	if (j%100==0)
          printf("\n");
        printf("%f ",hz[i][j]);
      }
      printf("\n");
    }
  }
#endif


  return ((int) hz[0][0]); 

}
                                    
